package fr.jelora.minitel;

import java.io.File;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;
import org.hibernate.Session;

import fr.jelora.jraspitools.RasPiPin;
import fr.jelora.jraspitools.gpio.GPIODirection;
import fr.jelora.jraspitools.gpio.GPIOTools;
import fr.jelora.minitel.db.hibernate.HibernateUtil;
import fr.jelora.minitel.enums.ModeEcran;
import fr.jelora.minitel.services.ServiceMinitel;
import fr.jelora.minitel.services.accueil.ServiceAccueil;
import fr.jelora.minitel.services.apropos.ServiceAPropos;
import fr.jelora.minitel.services.chat.ServiceChat;
import fr.jelora.minitel.services.demo.ServiceDemo;
import fr.jelora.minitel.services.echecs.ServiceEchecs;
import jssc.SerialPort;

public class ServeurMinitel {
	
	final static Logger logger = Logger.getLogger(ServeurMinitel.class);
	
	private String version = "0.1";
	private String dateVersion = "26/08/2017";
	private boolean modeDev = true;
	
	private String modemReception75;
	private String modemTransmission1200;
	
	private RasPiPin pinDecrochageLigne;
	private RasPiPin pinDetectionSonnerie;
	private RasPiPin pinActivationLiaisonDonnees;
	private RasPiPin pinDectectionConnexionMinitel;
	
	public static void main(String[] args) {
		ServeurMinitel serveurMinitel = new ServeurMinitel();
		serveurMinitel.run();
	}

	public void run(){
		logger.info("Serveur Minitel v" + version + " (" + dateVersion + ")");
		
		List<Object[]> services = new ArrayList<Object[]>();
		services.add(new Object[]{ "DEMO", "Démo utilisée dans la\nvidéo Teaser", ServiceDemo.class });
		services.add(new Object[]{ "ECHECS", "Jeu d'échecs", ServiceEchecs.class });
		services.add(new Object[]{ "APROPOS", "A propos de ce serveur", ServiceAPropos.class, version });
		services.add(new Object[]{ "CHAT", "Service de messagerie", ServiceChat.class});
		
		try{
			// Lecture configuration
			File fichierConfig = new File("config.properties");
			Properties config = new Properties();
			config.load(new FileInputStream(fichierConfig));
			if ( config.get("modeDev").equals("true") ){
				modeDev = true;
			}
			else{
				modeDev = false;
			}
			modemReception75 = config.getProperty("modemReception75");
			modemTransmission1200 = config.getProperty("modemTransmission1200");
			if ( config.get("pinDecrochageLigne") != null && !modeDev ){
				if ( !config.get("pinDecrochageLigne").toString().trim().isEmpty() ){
					pinDecrochageLigne = RasPiPin.valueOf(config.get("pinDecrochageLigne").toString().trim());
				}
			}
			if ( config.get("pinDetectionSonnerie") != null && !modeDev ){
				if ( !config.get("pinDetectionSonnerie").toString().trim().isEmpty() ){
					pinDetectionSonnerie = RasPiPin.valueOf(config.get("pinDetectionSonnerie").toString().trim());
				}
			}
			if ( config.get("pinActivationLiaisonDonnees") != null && !modeDev ){
				if ( !config.get("pinActivationLiaisonDonnees").toString().trim().isEmpty() ){
					pinActivationLiaisonDonnees = RasPiPin.valueOf(config.get("pinActivationLiaisonDonnees").toString().trim());
				}
			}
			if ( config.get("pinDectectionConnexionMinitel") != null && !modeDev ){
				if ( !config.get("pinDectectionConnexionMinitel").toString().trim().isEmpty() ){
					pinDectectionConnexionMinitel = RasPiPin.valueOf(config.get("pinDectectionConnexionMinitel").toString().trim());
				}
			}
			String dbServeur = null;
			String dbNom = null;
			String dbIdentifiant = null;
			String dbMotDePasse = null;
			if ( config.get("db.serveur") != null ){
				if ( !config.get("db.serveur").toString().trim().isEmpty() ){
					dbServeur = config.get("db.serveur").toString().trim();
				}
			}
			if ( config.get("db.nom") != null ){
				if ( !config.get("db.nom").toString().trim().isEmpty() ){
					dbNom = config.get("db.nom").toString().trim();
				}
			}
			if ( config.get("db.identifiant") != null ){
				if ( !config.get("db.identifiant").toString().trim().isEmpty() ){
					dbIdentifiant = config.get("db.identifiant").toString().trim();
				}
			}
			if ( config.get("db.motdepasse") != null ){
				if ( !config.get("db.motdepasse").toString().trim().isEmpty() ){
					dbMotDePasse = config.get("db.motdepasse").toString().trim();
				}
			}
			
			// Intialisation base de données
			HibernateUtil hibernateUtil = new HibernateUtil();
			Session session = hibernateUtil.buildSessionFactory(dbServeur, dbNom, dbIdentifiant, dbMotDePasse).openSession();
			
			// Initialisation des ports série
			SerialPort serialTransmission1200 = new SerialPort(modemTransmission1200);
			SerialPort serialReception75 = new SerialPort(modemReception75);
			
			// Initialisation des outils de communication
			MinitelTools minitelTools = new MinitelTools(serialTransmission1200, serialReception75);
			
			// Initialisation des GPIO
			GPIOTools gpioTools = null;
			if ( !modeDev ){
				gpioTools = new GPIOTools();
				gpioTools.init(pinDecrochageLigne, GPIODirection.out, false);
				gpioTools.init(pinDetectionSonnerie, GPIODirection.in);
				gpioTools.init(pinActivationLiaisonDonnees, GPIODirection.out, false);
				gpioTools.init(pinDectectionConnexionMinitel, GPIODirection.in);
			}
			
			// Vérification existance ports séries
			File fileModemReception75 = new File(modemReception75);
			File fileModemTransmission1200 = new File(modemTransmission1200);
			if ( !fileModemReception75.exists() ){
				logger.info("Port série \"" + modemReception75 + "\" non prêt.");
			}
			if ( !fileModemTransmission1200.exists() ){
				logger.info("Port série \"" + modemTransmission1200 + "\" non prêt.");
			}
			if ( !fileModemReception75.exists() || !fileModemTransmission1200.exists() ){
				logger.info("Attente ...");
			}
			while ( !fileModemReception75.exists() || !fileModemTransmission1200.exists() ){
				Thread.sleep(200);
			}
			logger.info("Prêt");
			
			
			// Mode normal: tourne en boucle pour décrocher à nouveau près une déconnexion
			// Mode dev: quitte après fin du traitement
			do {
				
				// Attente d'un appel
				if ( !modeDev ){
					logger.info("Attente appel ...");
					while ( !gpioTools.getValueBoolean(pinDetectionSonnerie) ){
						try {
							Thread.sleep(100);
						} 
						catch (InterruptedException e) {
							logger.debug(e);
						}
					}
				}
				
				// Décrochage ligne
				if ( !modeDev ){
					logger.info("Nouvel appel ! Décrochage.");
					gpioTools.setValue(pinDecrochageLigne, true);
				}
				
				// Activation liaison série au bout de 2 secondes
				if ( !modeDev ){
					Thread.sleep(2000);
					logger.info("Activation liaison série.");
					gpioTools.setValue(pinActivationLiaisonDonnees, true);
				}
				
				// Détection minitel
				// Raccrochage automatique au bout de 30 secondes si pas de connexion détectée
				boolean connexionEtablie = false;
				if ( !modeDev ){
					Thread.sleep(500);
					logger.info("Attente détection minitel ...");
					int verifConnexionMinitel = 0;
					int timeOutDetection = 0;
					while ( verifConnexionMinitel < 5 && timeOutDetection < 300 ){
						if ( !gpioTools.getValueBoolean(pinDectectionConnexionMinitel) ){
							verifConnexionMinitel = 0;
						}
						else{
							verifConnexionMinitel++;
						}
						try {
							Thread.sleep(100);
						} 
						catch (InterruptedException e) {
							logger.debug(e);
						}
						timeOutDetection++;
					}
					if ( timeOutDetection < 300 ){
						logger.info("Minitel connecté ! Connexion établie.");
						connexionEtablie = true;
					}
					else{
						logger.info("Tentative de connexion trop longue ... Pas de lancement de service et raccrochage.");
					}
				}
				else{
					// En mode développement, la connexion est toujours active
					connexionEtablie = true;
				}
				
				// Activation liaison série et lancement des services
				if ( connexionEtablie ){
					
					serialTransmission1200.openPort();
					serialTransmission1200.setParams(SerialPort.BAUDRATE_1200, SerialPort.DATABITS_7, SerialPort.STOPBITS_1, SerialPort.PARITY_EVEN);
					
					serialReception75.openPort();
					serialReception75.setParams(75, SerialPort.DATABITS_7, SerialPort.STOPBITS_1, SerialPort.PARITY_EVEN);
					
					minitelTools.signalerActiviteReception();
					minitelTools.demarrerThreadsSurveillance(pinDecrochageLigne, pinActivationLiaisonDonnees, pinDectectionConnexionMinitel, modeDev);
					
					try{
						// Service d'accueil
						ServiceMinitel serviceAccueil = new ServiceAccueil(services);
						String nomServiceDemande = serviceAccueil.run(minitelTools);
						//String nomServiceDemande = "CHAT";
						
						// Service demandé depuis le service d'accueil ou depuis un autre service
						while ( nomServiceDemande != null ){
							ServiceMinitel serviceDemande = null;
							for ( Object[] service : services ){
								if ( nomServiceDemande.toUpperCase().trim().equals((String) service[0]) ){
									Class<?> classeService = (Class<?>) service[2];
									serviceDemande = (ServiceMinitel) classeService.newInstance();
									serviceDemande.setSession(session);
									if ( service.length > 3 ){
										serviceDemande.setParametres(service[3]);
									}
									logger.info("Accès service " + nomServiceDemande.toUpperCase().trim());
									break;
								}
							}
							if ( serviceDemande != null ){
								nomServiceDemande = serviceDemande.run(minitelTools);
							}
							
							if ( nomServiceDemande != null ){
								if ( nomServiceDemande.trim().isEmpty() ){
									nomServiceDemande = null;
								}
							}
						}
						
						// Effacement écran après avoir quitté le service sans erreur
						if ( !modeDev ){
							minitelTools.changerModeEcran(ModeEcran.colonnes40);
							minitelTools.clearLigne00();
							minitelTools.clearScreen();
							minitelTools.setCursorBlinking(false);
							minitelTools.write("Au revoir !");
							minitelTools.locate(30, 23);
							minitelTools.write("Déconnexion");
						}
					}
					catch(Exception e){
						// Les erreurs ne sont pas prises en compte si la connexion vient d'être coupée
						if ( !modeDev ){
							if ( gpioTools.getValueBoolean(pinActivationLiaisonDonnees) && gpioTools.getValueBoolean(pinDecrochageLigne) ){
								logger.error(e);
								for ( int i = 0 ; i < e.getStackTrace().length ; i++ ){
									logger.error("	" + e.getStackTrace()[i]);
								}
								minitelTools.changerModeEcran(ModeEcran.colonnes40);
								minitelTools.clearLigne00();
								minitelTools.clearScreen();
								minitelTools.setCursorBlinking(false);
								minitelTools.write("Un erreur s'est produite. Veuillez réessayer plus tard.");
								minitelTools.locate(30, 23);
								minitelTools.write("Déconnexion");
							}
							else{
								// Exeception pour arrêter le service
							}
						}
						else{
							logger.error(e);
							for ( int i = 0 ; i < e.getStackTrace().length ; i++ ){
								logger.error("	" + e.getStackTrace()[i]);
							}
						}
					}
					
					minitelTools.arreterThreadsSurveillance();
					
					// Fermeture des ports si toujours ouverts
					if ( serialTransmission1200.isOpened() ){
						serialTransmission1200.closePort();
					}
					if ( serialReception75.isOpened() ){
						serialReception75.closePort();
					}
				}
				
				// Fermeture connexion série analogique et raccrochage ligne
				logger.info("Fin de session.");
				if ( !modeDev ){
					gpioTools.setValue(pinActivationLiaisonDonnees, false);
					gpioTools.setValue(pinDecrochageLigne, false);
				}
				
				Thread.sleep(100);
			}
			while ( !modeDev );
			
			
			// Fermeture session base de données
			session.close();
			
		}
		catch(Exception e){
			logger.error(e);
			for ( int i = 0 ; i < e.getStackTrace().length ; i++ ){
				logger.error("	" + e.getStackTrace()[i]);
			}
		}
		
		logger.info("Bye !");
	}
}
