package fr.jelora.minitel.services.accueil;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.apache.log4j.Logger;

import fr.jelora.minitel.MinitelTools;
import fr.jelora.minitel.enums.CodeCouleurCaractere;
import fr.jelora.minitel.enums.CodeCouleurDeFond;
import fr.jelora.minitel.enums.ModeEcran;
import fr.jelora.minitel.enums.TailleTexte;
import fr.jelora.minitel.enums.ToucheClavier;
import fr.jelora.minitel.enums.TypeToucheClavier;
import fr.jelora.minitel.exception.ReceptionException;
import fr.jelora.minitel.exception.TransmissionException;
import fr.jelora.minitel.services.ServiceMinitel;
import fr.jelora.minitel.services.ServiceMinitelAbstract;

public class ServiceAccueil extends ServiceMinitelAbstract implements ServiceMinitel {
	
	final static Logger logger = Logger.getLogger(ServiceAccueil.class);
	
	private MinitelTools minitelTools;
	private List<Object[]> services;
	private String serviceDemande;
	
	public ServiceAccueil(List<Object[]> services){
		this.services = services;
	}
	
	public String run(MinitelTools minitelTools) throws TransmissionException, ReceptionException {
		this.minitelTools = minitelTools;
		serviceDemande = null;
		
		minitelTools.changerModeEcran(ModeEcran.colonnes40);
		minitelTools.changerModeEcran(ModeEcran.page);
		
		Affichage affichage = Affichage.accueil;
		while ( affichage != null ){
			if ( affichage == Affichage.accueil ){
				affichage = accueil();
			}
			if ( affichage == Affichage.listeDesServices ){
				affichage = listeDesServices();
			}
		}
		
		return serviceDemande;
	}
	
	private enum Affichage {
		accueil,
		listeDesServices
	}
	
	private Affichage accueil() throws TransmissionException, ReceptionException{
		
		minitelTools.clearAll();
		
		minitelTools.writeImage(1, 1, new File("serveur.png"));
		
		minitelTools.locate(8, 1);
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris4);
		minitelTools.setColorText(CodeCouleurCaractere.caractereGris7);
		minitelTools.write(" Serveur Minitel ");
		
		minitelTools.locate(10, 3);
		minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
		minitelTools.setColorText(CodeCouleurCaractere.caractereGris2);
		minitelTools.write(" de ");
		minitelTools.setTailleTexte(TailleTexte.doubleHauteur);
		minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
		minitelTools.write(" Jelora ");
		
		minitelTools.setTailleTexte(TailleTexte.normal);
		minitelTools.locate(2, 9);
		minitelTools.write("Entrez le code du service auquel vous");
		minitelTools.locate(2, 10);
		minitelTools.write("souhaitez accéder:");
		minitelTools.locate(1, 12);
		minitelTools.write("CODE:");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris2);
		minitelTools.write(" ");
		minitelTools.writeRepeteByte((byte) '.', 30);
		minitelTools.write(" ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
		minitelTools.locate(16, 14);
		minitelTools.write("puis appuyez sur ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris6);
		minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
		minitelTools.write(" Envoi ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
		minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
		
		minitelTools.locate(1, 20);
		minitelTools.writeRepeteByte((byte) 0x60, 39);
		
		minitelTools.locate(24, 21);
		minitelTools.write("Effacer ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris6);
		minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
		minitelTools.write(" Correc. ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
		minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
		
		minitelTools.locate(19, 22);
		minitelTools.write("Effacer tout ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris6);
		minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
		minitelTools.write(" Annul.  ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
		minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
		
		minitelTools.locate(4, 24);
		minitelTools.write("Liste des codes de services ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris6);
		minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
		minitelTools.write(" Guide   ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
		minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
		
		
		minitelTools.locate(7, 12);
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris2);
		minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
		minitelTools.setCursorBlinking(true);
		
		minitelTools.viderCacheRead();
		Affichage affichageSuivant = null;
		serviceDemande = "";
		boolean messageServiceInexistant = false;
		while ( true ){
			ToucheClavier toucheClavier = minitelTools.readClavier();
			if ( toucheClavier != null ){
				//logger.info(toucheClavier.getTypeToucheClavier().name() + ": " + toucheClavier.name());
				
				if ( messageServiceInexistant ){
					minitelTools.locate(3, 17);
					minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
					minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
					minitelTools.write(" ");
					minitelTools.writeRepeteByte((byte) ' ', 18);
					
					minitelTools.locate(7 + serviceDemande.length(), 12);
					messageServiceInexistant = false;
				}
				
				if ( toucheClavier.getTypeToucheClavier() == TypeToucheClavier.Caractere ){
					if ( serviceDemande.length() < 31 ){
						serviceDemande += toucheClavier.toString();
						minitelTools.write(toucheClavier.toString());
					}
					else{
						minitelTools.faireUnBip();
					}
				}
				else if ( toucheClavier == ToucheClavier.ConnexionFin ){
					serviceDemande = null;
					break;
				}
				else if ( toucheClavier == ToucheClavier.Guide ){
					serviceDemande = null;
					affichageSuivant = Affichage.listeDesServices;
					break;
				}
				else if ( toucheClavier == ToucheClavier.Annulation ){
					serviceDemande = "";
					minitelTools.locate(6, 12);
					minitelTools.setColorBackground(CodeCouleurDeFond.fondGris2);
					minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
					minitelTools.write(" ");
					minitelTools.writeRepeteByte((byte) '.', 30);
					minitelTools.write(" ");
					minitelTools.locate(7, 12);
				}
				else if ( toucheClavier == ToucheClavier.Correction || toucheClavier == ToucheClavier.ToucheGauche ){
					if ( serviceDemande.length() >= 1 ){
						serviceDemande = serviceDemande.substring(0, serviceDemande.length() - 1);
						minitelTools.locate(7 + serviceDemande.length(), 12);
						minitelTools.write(".");
						minitelTools.locate(7 + serviceDemande.length(), 12);
					}
					else{
						minitelTools.faireUnBip();
					}
				}
				else if ( toucheClavier == ToucheClavier.Envoi ){
					if ( serviceDemande.length() >= 1 ){
						boolean serviceExistant = false;
						for ( Object[] service : services ){
							if ( serviceDemande.toUpperCase().trim().equals((String) service[0]) ){
								serviceExistant = true;
							}
						}
						
						if ( serviceExistant ){
							break;
						}
						else{
							minitelTools.locate(3, 17);
							minitelTools.setColorBackground(CodeCouleurDeFond.fondGris5);
							minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
							minitelTools.write(" ");
							minitelTools.write("Service inexistant ");
							messageServiceInexistant = true;
							
							serviceDemande = "";
							minitelTools.locate(6, 12);
							minitelTools.setColorBackground(CodeCouleurDeFond.fondGris2);
							minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
							minitelTools.write(" ");
							minitelTools.writeRepeteByte((byte) '.', 30);
							minitelTools.write(" ");
							minitelTools.locate(7, 12);
						}
					}
					else{
						minitelTools.faireUnBip();
					}
				}
				else{
					minitelTools.faireUnBip();
				}
			}
		}
		
		return affichageSuivant;
	}
	
	private Affichage listeDesServices() throws TransmissionException, ReceptionException {
		
		minitelTools.clearAll();
		
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris4);
		minitelTools.setColorText(CodeCouleurCaractere.caractereGris7);
		minitelTools.write(" Liste des codes de services");
		minitelTools.writeRepeteByte((byte) ' ', 11);
		minitelTools.locate(1, 2);
		minitelTools.writeRepeteByte((byte) 0x60, 39);
		
		minitelTools.locate(1, 20);
		minitelTools.writeRepeteByte((byte) 0x60, 39);
		
		minitelTools.locate(19, 21);
		minitelTools.write("Page suivante ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris6);
		minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
		minitelTools.write(" Suite  ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
		minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
		
		minitelTools.locate(17, 22);
		minitelTools.write("Page précédente ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris6);
		minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
		minitelTools.write(" Retour ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
		minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
		
		minitelTools.locate(25, 24);
		minitelTools.write("Accueil ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris6);
		minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
		minitelTools.write(" Somm.  ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
		minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
		
		minitelTools.locate(1, 21);
		minitelTools.write("Page");
		
		minitelTools.locate(2, 3);
		minitelTools.setColorText(CodeCouleurCaractere.caractereGris4);
		minitelTools.write("Code");
		minitelTools.locate(15, 3);
		minitelTools.write("Description");
		
		minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
		
		// Tri des services par ordre alphabétique
		// Création d'un nouvel objet pour ne pas modifier l'objet original
		List<Object[]> servicesTries = new ArrayList<Object[]>(services);
		Collections.sort(servicesTries, new Comparator<Object[]>() {
			@Override
			public int compare(Object[] service1, Object[] service2) {
				return ((String) service1[0]).toLowerCase().compareTo(((String) service2[0]).toLowerCase());
			}
		});
		
		// Préparation des pages
		// 14 lignes max par page
		List<PageListeDesServices> pages = new ArrayList<PageListeDesServices>();
		PageListeDesServices page = new PageListeDesServices();
		int nbLignesPage = 0;
		String code = null;
		String description = null;
		for ( Object[] service : servicesTries ){
			code = (String) service[0];
			description = (String) service[1];
			
			if ( nbLignesPage + description.split("\n").length + 1 > 15 ){
				pages.add(page);
				page = new PageListeDesServices();
				nbLignesPage = 0;
			}
			
			nbLignesPage += description.split("\n").length + 1;
			page.services.add(new String[]{code, description});
		}
		pages.add(page);
		minitelTools.locate(8, 21);
		minitelTools.write("/ " + pages.size());
		
		// Affichage par page
		int idPage = 1;
		int idLigne = 1;
		String[] lignesDescription;
		boolean continuer = true;
		Affichage affichageSuivant = null;
		while ( continuer ){
			idLigne = 0;
			
			
			minitelTools.locate(6, 21);
			minitelTools.write(idPage + "");
			
			for ( String[] service : pages.get(idPage - 1).services ){
				minitelTools.locate(2, 5 + idLigne);
				minitelTools.write(service[0].toUpperCase());
				
				lignesDescription = service[1].split("\n");
				for ( int i = 0 ; i < lignesDescription.length ; i++ ){
					minitelTools.locate(15, 5 + idLigne);
					minitelTools.write(lignesDescription[i]);
					idLigne++;
				}
				
				idLigne++;
			}
			
			while ( true ){
				ToucheClavier toucheClavier = minitelTools.readClavier();
				if ( toucheClavier != null ){
					
					if ( toucheClavier == ToucheClavier.ConnexionFin ){
						continuer = false;
						break;
					}
					else if ( toucheClavier == ToucheClavier.Sommaire ){
						continuer = false;
						affichageSuivant = Affichage.accueil;
						break;
					}
					else if ( toucheClavier == ToucheClavier.Suite ){
						if ( idPage < pages.size() ){
							minitelTools.locate(6, 21);
							minitelTools.write("  ");
							for ( int i = 0 ; i < 15 ; i++ ){
								minitelTools.locate(1, 5 + i);
								minitelTools.writeRepeteByte((byte) ' ', 39);
							}
							
							idPage++;
							break;
						}
						else{
							minitelTools.faireUnBip();
						}
					}
					else if ( toucheClavier == ToucheClavier.Retour ){
						if ( idPage > 1 ){
							minitelTools.locate(6, 21);
							minitelTools.write("  ");
							for ( int i = 0 ; i < 15 ; i++ ){
								minitelTools.locate(1, 5 + i);
								minitelTools.writeRepeteByte((byte) ' ', 39);
							}
							
							idPage--;
							break;
						}
						else{
							minitelTools.faireUnBip();
						}
					}
					else{
						minitelTools.faireUnBip();
					}
				}
			}
		}
		
		
		return affichageSuivant;
	}
	
	private class PageListeDesServices {
		List<String[]> services;
		public PageListeDesServices(){
			this.services = new ArrayList<String[]>();
		}
	}
}
