package fr.jelora.minitel.services.apropos;

import java.io.File;

import fr.jelora.minitel.MinitelTools;
import fr.jelora.minitel.enums.CodeCouleurCaractere;
import fr.jelora.minitel.enums.CodeCouleurDeFond;
import fr.jelora.minitel.enums.ModeEcran;
import fr.jelora.minitel.enums.ToucheClavier;
import fr.jelora.minitel.exception.ReceptionException;
import fr.jelora.minitel.exception.TransmissionException;
import fr.jelora.minitel.services.ServiceMinitel;
import fr.jelora.minitel.services.ServiceMinitelAbstract;

public class ServiceAPropos extends ServiceMinitelAbstract implements ServiceMinitel {
	
	private MinitelTools minitelTools;
	private String version;
	
	public void setParametres(Object parametres){
		this.version = (String) parametres;
	}
	
	public String run(MinitelTools minitelTools) throws TransmissionException, ReceptionException {
		this.minitelTools = minitelTools;
		
		Affichage affichage = Affichage.accueil;
		while ( affichage != null ){
			if ( affichage == Affichage.accueil ){
				affichage = accueil();
			}
			if ( affichage == Affichage.quiEstJelora ){
				affichage = quiEstJelora();
			}
		}
		
		return null;
	}
	
	private enum Affichage {
		accueil,
		quiEstJelora
	}
	
	private Affichage accueil() throws TransmissionException, ReceptionException{
		
		minitelTools.changerModeEcran(ModeEcran.colonnes40);
		minitelTools.clearAll();
		minitelTools.write("A propos ...");
		minitelTools.locate(1, 2);
		minitelTools.writeRepeteByte((byte) 0x60, 39);
		
		minitelTools.locate(2, 4);
		minitelTools.write("Serveur Minitel v" + version);
		minitelTools.locate(2, 5);
		minitelTools.write("Mis en ligne le 27/08/2017");
		
		minitelTools.locate(2, 7);
		minitelTools.write("Logiciel écrit en Java");
		minitelTools.locate(2, 8);
		minitelTools.write("Fonctionne sur un RaspberryPi 3");
		minitelTools.locate(2, 9);
		minitelTools.write("sous Raspbian");
		minitelTools.locate(2, 10);
		minitelTools.write("Conçu et réalisé par Jelora");
		
		minitelTools.locate(1, 22);
		minitelTools.writeRepeteByte((byte) 0x60, 39);
		
		minitelTools.locate(24, 23);
		minitelTools.write("Quitter ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris6);
		minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
		minitelTools.write(" ConnFin ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
		minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
		
		minitelTools.locate(15, 24);
		minitelTools.write("Qui est Jelora ? ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris6);
		minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
		minitelTools.write(" Guide   ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
		minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
		
		Affichage affichageSuivant = null;
		while ( true ){
			ToucheClavier toucheClavier = minitelTools.readClavier();
			if ( toucheClavier != null ){
				if ( toucheClavier == ToucheClavier.Guide ){
					affichageSuivant = Affichage.quiEstJelora;
					break;
				}
				else if ( toucheClavier == ToucheClavier.ConnexionFin ){
					break;
				}
				else{
					minitelTools.faireUnBip();
				}
			}
		}
		
		return affichageSuivant;
	}
	
	private Affichage quiEstJelora() throws TransmissionException, ReceptionException{
		
		minitelTools.changerModeEcran(ModeEcran.colonnes40);
		minitelTools.clearAll();
		
		minitelTools.locate(2, 3);
		minitelTools.write("Jelora");
		
		minitelTools.writeImage(14, 2, new File("jelora.jpg"));
		
		minitelTools.locate(1, 5);
		minitelTools.write("Mec qui aime bien");
		minitelTools.locate(1, 6);
		minitelTools.write("bricoler un peu");
		minitelTools.locate(1, 7);
		minitelTools.write("de tout en amateur");
		
		minitelTools.locate(1, 19);
		minitelTools.write("Site web: http://www.jelora.fr");
		minitelTools.locate(1, 20);
		minitelTools.write("Twitter: @jelora119");
		minitelTools.locate(1, 21);
		minitelTools.write("Mastodon: @jelora@mastodon.gargantia.fr");
		
		minitelTools.locate(1, 22);
		minitelTools.writeRepeteByte((byte) 0x60, 39);
		
		minitelTools.locate(24, 24);
		minitelTools.write("A propos ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris6);
		minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
		minitelTools.write(" Retour ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
		minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
		
		Affichage affichageSuivant = null;
		while ( true ){
			ToucheClavier toucheClavier = minitelTools.readClavier();
			if ( toucheClavier != null ){
				if ( toucheClavier == ToucheClavier.Retour ){
					affichageSuivant = Affichage.accueil;
					break;
				}
				else if ( toucheClavier == ToucheClavier.ConnexionFin ){
					break;
				}
				else{
					minitelTools.faireUnBip();
				}
			}
		}
		
		return affichageSuivant;
	}
}
