package fr.jelora.minitel.services.chat;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;

import org.apache.log4j.Logger;
import org.hibernate.Query;

import fr.jelora.minitel.MinitelTools;
import fr.jelora.minitel.db.entity.MessageChat;
import fr.jelora.minitel.enums.CodeCouleurCaractere;
import fr.jelora.minitel.enums.CodeCouleurDeFond;
import fr.jelora.minitel.enums.ModeEcran;
import fr.jelora.minitel.enums.ToucheClavier;
import fr.jelora.minitel.enums.TypeToucheClavier;
import fr.jelora.minitel.exception.ReceptionException;
import fr.jelora.minitel.exception.TransmissionException;
import fr.jelora.minitel.services.ServiceMinitel;
import fr.jelora.minitel.services.ServiceMinitelAbstract;

@SuppressWarnings("unchecked")
public class ServiceChat extends ServiceMinitelAbstract implements ServiceMinitel {
	
	final static Logger logger = Logger.getLogger(ServiceChat.class);
	
	private MinitelTools minitelTools;
	private String identifiant = "";
	
	public String run(MinitelTools minitelTools) throws TransmissionException, ReceptionException {
		this.minitelTools = minitelTools;
		
		minitelTools.changerModeEcran(ModeEcran.colonnes40);
		minitelTools.changerModeEcran(ModeEcran.page);
		
		Affichage affichage = Affichage.accueil;
		while ( affichage != null ){
			if ( affichage == Affichage.accueil ){
				affichage = accueil();
			}
			if ( affichage == Affichage.messagesChat ){
				affichage = messagesChat();
			}
		}
		
		return null;
	}
	
	private enum Affichage {
		accueil,
		messagesChat
	}
	
	private Affichage accueil() throws TransmissionException, ReceptionException{
		
		minitelTools.changerModeEcran(ModeEcran.page);
		
		minitelTools.clearAll();
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris4);
		minitelTools.write(" Chat");
		minitelTools.writeRepeteByte((byte) ' ', 34);
		minitelTools.locate(1, 2);
		minitelTools.writeRepeteByte((byte) 0x60, 39);
		
		minitelTools.locate(1, 22);
		minitelTools.writeRepeteByte((byte) 0x60, 39);
		
		minitelTools.locate(24, 24);
		minitelTools.write("Quitter ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris6);
		minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
		minitelTools.write(" ConnFin ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
		minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
		
		minitelTools.locate(2, 17);
		minitelTools.write("Commandes:");
		minitelTools.locate(2, 18);
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris6);
		minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
		minitelTools.write("  Envoi  ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
		minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
		minitelTools.write(" Taper/Envoyer message");
		minitelTools.locate(2, 19);
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris6);
		minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
		minitelTools.write("  Répét. ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
		minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
		minitelTools.write(" Réactualiser");
		minitelTools.locate(2, 20);
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris6);
		minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
		minitelTools.write(" ConnFin ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
		minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
		minitelTools.write(" Quitter");
		
		minitelTools.locate(2, 4);
		minitelTools.write("Identifiant:");
		
		Affichage affichageSuivant = null;
		identifiant = "";
		boolean boucleIdentifiant = true;
		while ( boucleIdentifiant ){
			
			minitelTools.locate(2, 5);
			minitelTools.setColorBackground(CodeCouleurDeFond.fondGris2);
			minitelTools.write(" ");
			minitelTools.writeRepeteByte((byte) '.', 19);
			minitelTools.write(" ");
			minitelTools.locate(3, 5);
			minitelTools.setColorBackground(CodeCouleurDeFond.fondGris2);
			minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
			minitelTools.setCursorBlinking(true);
			
			minitelTools.viderCacheRead();
			while ( true ){
				ToucheClavier toucheClavier = minitelTools.readClavier();
				if ( toucheClavier != null ){
					
					if ( toucheClavier.getTypeToucheClavier() == TypeToucheClavier.Caractere && toucheClavier != ToucheClavier.Espace ){
						if ( identifiant.length() < 20 ){
							identifiant += toucheClavier.toString();
							minitelTools.write(toucheClavier.toString());
						}
						else{
							minitelTools.faireUnBip();
						}
					}
					else if ( toucheClavier == ToucheClavier.ConnexionFin ){
						boucleIdentifiant = false;
						identifiant = "";
						break;
					}
					else if ( toucheClavier == ToucheClavier.Annulation ){
						identifiant = "";
						minitelTools.locate(3, 5);
						minitelTools.writeRepeteByte((byte) '.', 19);
						minitelTools.locate(3, 5);
						minitelTools.setColorBackground(CodeCouleurDeFond.fondGris2);
					}
					else if ( toucheClavier == ToucheClavier.Correction || toucheClavier == ToucheClavier.ToucheGauche ){
						if ( identifiant.length() >= 1 ){
							identifiant = identifiant.substring(0, identifiant.length() - 1);
							minitelTools.locate(3 + identifiant.length(), 5);
							minitelTools.write(".");
							minitelTools.locate(3 + identifiant.length(), 5);
							minitelTools.setColorBackground(CodeCouleurDeFond.fondGris2);
						}
						else{
							minitelTools.faireUnBip();
						}
					}
					else if ( toucheClavier == ToucheClavier.Envoi ){
						if ( identifiant.length() >= 1 ){
							minitelTools.setCursorBlinking(false);
							break;
						}
						else{
							minitelTools.faireUnBip();
						}
					}
					else{
						minitelTools.faireUnBip();
					}
				}
			}
			
			if ( !identifiant.isEmpty() ){
				
				// Vérification si l'identifiant est protégé
				// Si oui, demande de mot de passe
				// Si non, affichage du chat
				
				if ( identifiantProtege(identifiant) ){
					minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
					minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
					
					minitelTools.locate(2, 7);
					minitelTools.write("Identifiant protégé");
					minitelTools.locate(2, 10);
					minitelTools.write("Mot de passe:");
					
					minitelTools.locate(23, 23);
					minitelTools.write("Modifier ");
					minitelTools.setColorBackground(CodeCouleurDeFond.fondGris6);
					minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
					minitelTools.write(" Retour  ");
					minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
					minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
					
					boolean boucleMotDePasse = true;
					while ( boucleMotDePasse ){
						
						minitelTools.locate(2, 11);
						minitelTools.setColorBackground(CodeCouleurDeFond.fondGris2);
						minitelTools.write(" ");
						minitelTools.writeRepeteByte((byte) '.', 29);
						minitelTools.write(" ");
						minitelTools.locate(3, 11);
						minitelTools.setColorBackground(CodeCouleurDeFond.fondGris2);
						minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
						minitelTools.setCursorBlinking(true);
						
						minitelTools.viderCacheRead();
						String motDePasse = "";
						while ( true ){
							ToucheClavier toucheClavier = minitelTools.readClavier();
							if ( toucheClavier != null ){
								
								if ( toucheClavier.getTypeToucheClavier() == TypeToucheClavier.Caractere && toucheClavier != ToucheClavier.Espace ){
									if ( motDePasse.length() < 30 ){
										motDePasse += toucheClavier.toString();
										minitelTools.write("*");
									}
									else{
										minitelTools.faireUnBip();
									}
								}
								else if ( toucheClavier == ToucheClavier.ConnexionFin ){
									boucleIdentifiant = false;
									identifiant = "";
									motDePasse = "";
									break;
								}
								else if ( toucheClavier == ToucheClavier.Annulation ){
									motDePasse = "";
									minitelTools.locate(3, 11);
									minitelTools.writeRepeteByte((byte) '.', 29);
									minitelTools.locate(3, 11);
									minitelTools.setColorBackground(CodeCouleurDeFond.fondGris2);
								}
								else if ( toucheClavier == ToucheClavier.Correction || toucheClavier == ToucheClavier.ToucheGauche ){
									if ( motDePasse.length() >= 1 ){
										motDePasse = motDePasse.substring(0, motDePasse.length() - 1);
										minitelTools.locate(3 + motDePasse.length(), 11);
										minitelTools.write(".");
										minitelTools.locate(3 + motDePasse.length(), 11);
										minitelTools.setColorBackground(CodeCouleurDeFond.fondGris2);
									}
									else{
										minitelTools.faireUnBip();
									}
								}
								else if ( toucheClavier == ToucheClavier.Envoi ){
									if ( motDePasse.length() >= 1 ){
										minitelTools.setCursorBlinking(false);
										break;
									}
									else{
										minitelTools.faireUnBip();
									}
								}
								else if ( toucheClavier == ToucheClavier.Retour ){
									identifiant = "";
									motDePasse = "";
									minitelTools.locate(2, 7);
									minitelTools.clearLigneDuCurseurALaFinDeLaLigne();
									minitelTools.locate(2, 10);
									minitelTools.clearLigneDuCurseurALaFinDeLaLigne();
									minitelTools.locate(2, 11);
									minitelTools.clearLigneDuCurseurALaFinDeLaLigne();
									minitelTools.locate(23, 23);
									minitelTools.clearLigneDuCurseurALaFinDeLaLigne();
									boucleMotDePasse = false;
									break;
								}
								else{
									minitelTools.faireUnBip();
								}
							}
						}
						
						// Vérification
						if ( verifierIdentification(identifiant, motDePasse) ){
							// OK
							boucleIdentifiant = false;
							affichageSuivant = Affichage.messagesChat;
							break;
						}
						else{
							// Mauvais mot de passe, recommence
							minitelTools.faireUnBip();
						}
					}
					
				}
				else{
					// Identifiant non protégé
					// Peut être utilisé dans le chat directement
					boucleIdentifiant = false;
					affichageSuivant = Affichage.messagesChat;
				}
			}
		}
		
		return affichageSuivant;
	}
	
	private boolean verifierIdentification(String identifiant, String motDePasse){
		boolean result = false;
		if ( identifiant != null && motDePasse != null ){
			if ( !identifiant.trim().isEmpty() && !motDePasse.trim().isEmpty() ){
				String motDePasseMD5 = null;
				try{
					motDePasseMD5 = this.getStringMD5FromString(motDePasse);
				}
				catch(Exception e){
					logger.error(e);
				}
				if ( motDePasseMD5 != null ){
					Query requete = session.createQuery("SELECT COUNT(c) FROM CompteChat c WHERE LOWER(c.identifiant) = :identifiant AND c.motDePasseMD5 = :motDePasseMD5");
					requete.setParameter("identifiant", identifiant.trim().toLowerCase());
					requete.setParameter("motDePasseMD5", motDePasseMD5);
					List<Long> resultLong = requete.list();
					if ( resultLong != null ){
						if ( resultLong.size() > 0 ){
							if ( resultLong.get(0) > 0 ){
								result = true;
							}
						}
					}
				}
			}
		}
		return result;
	}
	
	private boolean identifiantProtege(String identifiant){
		boolean result = false;
		if ( identifiant != null ){
			if ( !identifiant.trim().isEmpty() ){
				Query requete = session.createQuery("SELECT COUNT(c) FROM CompteChat c WHERE LOWER(c.identifiant) = :identifiant");
				requete.setParameter("identifiant", identifiant.trim().toLowerCase());
				List<Long> resultLong = requete.list();
				if ( resultLong != null ){
					if ( resultLong.size() > 0 ){
						if ( resultLong.get(0) > 0 ){
							result = true;
						}
					}
				}
			}
		}
		return result;
	}
	
	private Affichage messagesChat() throws TransmissionException, ReceptionException{
		
		minitelTools.clearAll();
		minitelTools.changerModeEcran(ModeEcran.rouleau);
		
		minitelTools.allerDansLigne00(1);
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris4);
		minitelTools.write(" [Chat] " + this.identifiant + "\n");
		
		// Affichage des anciens messages
		List<MessageChat> derniersMessages = this.recupererDerniersMessages(12);
		for ( MessageChat messageChat : derniersMessages ){
			afficherMessageChat(messageChat);
		}
		
		try {
			Thread.sleep(100);
		} 
		catch (InterruptedException e) {
			throw new TransmissionException(e);
		}
		
		minitelTools.viderCacheRead();
		
		Affichage affichageSuivant = null;
		boolean ecritureDUnMessage = false;
		String message = "";
		while ( true ){
			ToucheClavier toucheClavier = minitelTools.readClavier();
			if ( toucheClavier != null ){
				
				if ( toucheClavier.getTypeToucheClavier() == TypeToucheClavier.Caractere ){
					if ( ecritureDUnMessage ){
						if ( message.length() < 120 ){
							message += toucheClavier.toString();
							minitelTools.write(toucheClavier.toString());
						}
						else{
							minitelTools.faireUnBip();
						}
					}
					else{
						minitelTools.faireUnBip();
					}
				}
				else if ( toucheClavier == ToucheClavier.Correction ){
					if ( message.length() > 0 ){
						message = message.substring(0, message.length()-1);
						minitelTools.write(ToucheClavier.ToucheGauche.toByteArray());
						minitelTools.write(" ");
						minitelTools.write(ToucheClavier.ToucheGauche.toByteArray());
					}
					else{
						minitelTools.faireUnBip();
					}
				}
				else if ( toucheClavier == ToucheClavier.Annulation ){
					message = "";
					minitelTools.setCursorBlinking(false);
					minitelTools.write("- Aucun message envoyé -\n\r\n\r");
					ecritureDUnMessage = false;
				}
				else if ( toucheClavier == ToucheClavier.Envoi ){
					if ( !ecritureDUnMessage ){
						minitelTools.write("\n\r");
						minitelTools.write("Tapez votre message: (180 carac. max)\n\r");
						minitelTools.setCursorBlinking(true);
						ecritureDUnMessage = true;
					}
					else{
						minitelTools.setCursorBlinking(false);
						if ( !message.trim().isEmpty() && message.length() < 180 ){
							// Enregistrement du message et affichage
							MessageChat nouveauMessage = enregistrerNouveauMessage(identifiant, message.trim());
							minitelTools.write("\n\r");
							minitelTools.write("\n\r");
							if ( nouveauMessage != null ){
								afficherMessageChat(nouveauMessage);
							}
							else{
								minitelTools.write("Aucun message envoyé -\n\r\n\r");
							}
							message = "";
						}
						else{
							minitelTools.write("- Aucun message envoyé -\n\r\n\r");
						}
						ecritureDUnMessage = false;
					}
				}
				else if ( toucheClavier == ToucheClavier.Repetition ){
					affichageSuivant = Affichage.messagesChat;
					break;
				}
				else if ( toucheClavier == ToucheClavier.ConnexionFin ){
					break;
				}
				else{
					minitelTools.faireUnBip();
				}
			}
		}
		
		
		return affichageSuivant;
	}
	
	private void afficherMessageChat(MessageChat messageChat) throws TransmissionException{
		if ( messageChat != null ){
			if ( messageChat.getDateHeure() != null && messageChat.getAuteur() != null && messageChat.getContenu() != null ){
				DateFormat formatter = new SimpleDateFormat("dd/MM HH:mm");
				
				minitelTools.setColorBackground(CodeCouleurDeFond.fondGris7);
				minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
				minitelTools.write(" " + formatter.format(messageChat.getDateHeure()) + " ");
				minitelTools.setColorBackground(CodeCouleurDeFond.fondGris2);
				minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
				minitelTools.write(" " + messageChat.getAuteur() + " ");
				minitelTools.writeRepeteByte((byte) ' ', 24 - messageChat.getAuteur().length());
				minitelTools.write(messageChat.getContenu() + "\n\r");
			}
		}
	}
	
	private MessageChat enregistrerNouveauMessage(String auteur, String contenu){
		MessageChat messageChat = null;
		if ( auteur != null && contenu != null ){
			if ( !auteur.trim().isEmpty() && !contenu.trim().isEmpty() ){
				messageChat = new MessageChat();
				messageChat.setDateHeure(new Date());
				messageChat.setAuteur(auteur);
				messageChat.setContenu(contenu);
				
				session.persist(messageChat);
				session.flush();
			}
		}
		return messageChat;
	}
	
	private List<MessageChat> recupererDerniersMessages(int nbMessages){
		Query requete = session.createQuery("SELECT m FROM MessageChat m ORDER BY m.idMessageChat DESC");
		requete.setMaxResults(nbMessages);
		List<MessageChat> messages = requete.list();
		
		if ( messages != null ){
			Collections.sort(messages, new Comparator<MessageChat>() {
				@Override
				public int compare(MessageChat message1, MessageChat message2) {
					return Integer.compare(message1.getIdMessageChat(), message2.getIdMessageChat());
				}
			});
		}
		
		if ( messages == null ){
			messages = new ArrayList<MessageChat>();
		}
		return messages;
	}
}
