package fr.jelora.minitel.services.echecs;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Random;
import java.util.concurrent.Semaphore;

import org.apache.log4j.Logger;

import fr.jelora.minitel.MinitelTools;
import fr.jelora.minitel.enums.CodeCouleurCaractere;
import fr.jelora.minitel.enums.CodeCouleurDeFond;
import fr.jelora.minitel.enums.ModeEcran;
import fr.jelora.minitel.enums.ToucheClavier;
import fr.jelora.minitel.enums.TypeToucheClavier;
import fr.jelora.minitel.exception.ReceptionException;
import fr.jelora.minitel.exception.TransmissionException;
import fr.jelora.minitel.services.ServiceMinitel;
import fr.jelora.minitel.services.ServiceMinitelAbstract;

public class ServiceEchecs extends ServiceMinitelAbstract implements ServiceMinitel {
	
	final static Logger logger = Logger.getLogger(ServiceEchecs.class);
	
	private MinitelTools minitelTools;
	private Difficulte difficulte;
	private Camp campJoueur;
	private PieceJeu[][] plateau;
	
	public String run(MinitelTools minitelTools) throws TransmissionException, ReceptionException {
		this.minitelTools = minitelTools;
		
		minitelTools.changerModeEcran(ModeEcran.colonnes40);
		
		Affichage affichage = Affichage.accueil;
		while ( affichage != null ){
			if ( affichage == Affichage.accueil ){
				affichage = accueil();
			}
			if ( affichage == Affichage.partie ){
				affichage = partie();
			}
		}
		
		return null;
	}
	
	private enum Affichage {
		accueil,
		partie
	}
	
	private Affichage accueil() throws TransmissionException, ReceptionException{
		
		minitelTools.clearAll();
		minitelTools.write("Jeu d'échecs");
		minitelTools.locate(1, 2);
		minitelTools.writeRepeteByte((byte) 0x60, 39);
		
		minitelTools.locate(1, 20);
		minitelTools.writeRepeteByte((byte) 0x60, 39);
		
		minitelTools.locate(24, 23);
		minitelTools.write("Quitter ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris6);
		minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
		minitelTools.write(" ConnFin ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
		minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
		
		minitelTools.locate(24, 21);
		minitelTools.write("Effacer ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris6);
		minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
		minitelTools.write(" Correc. ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
		minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
		
		minitelTools.locate(2, 4);
		minitelTools.write("Choisissez votre camp:");
		minitelTools.locate(4, 5);
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris7);
		minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
		minitelTools.write(" N ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
		minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
		minitelTools.write(" Les noirs");
		minitelTools.locate(4, 6);
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris7);
		minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
		minitelTools.write(" B ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
		minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
		minitelTools.write(" Les blancs");
		
		minitelTools.locate(2, 8);
		minitelTools.write("Choix: ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris2);
		minitelTools.write(" . ");
		
		Affichage affichageSuivant = null;
		boolean boucleParametresJeu = true;
		while ( boucleParametresJeu ){
			
			// Choix camp
			campJoueur = null;
			minitelTools.locate(10, 8);
			minitelTools.write(".");
			minitelTools.locate(13, 8);
			minitelTools.setColorBackground(CodeCouleurDeFond.fondGris6);
			minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
			minitelTools.write(" Envoi ");
			
			minitelTools.locate(10, 8);
			minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
			minitelTools.setCursorBlinking(true);
			
			while ( true ){
				ToucheClavier toucheClavier = minitelTools.readClavier();
				if ( toucheClavier != null ){
					
					if ( toucheClavier == ToucheClavier.N || toucheClavier == ToucheClavier.n || toucheClavier == ToucheClavier.B || toucheClavier == ToucheClavier.b ){
						if ( campJoueur == null ){
							if ( toucheClavier.toString().toUpperCase().equals("N") ){
								campJoueur = Camp.noir;
							}
							else{
								campJoueur = Camp.blanc;
							}
							minitelTools.write(toucheClavier.toString().toUpperCase());
						}
						else{
							minitelTools.faireUnBip();
						}
					}
					else if ( toucheClavier == ToucheClavier.Correction ){
						if ( campJoueur != null ){
							campJoueur = null;
							minitelTools.locate(10, 8);
							minitelTools.write(".");
							minitelTools.locate(10, 8);
						}
						else{
							minitelTools.faireUnBip();
						}
					}
					else if ( toucheClavier == ToucheClavier.Envoi || toucheClavier == ToucheClavier.Entree ){
						if ( campJoueur != null ){
							break;
						}
						else{
							minitelTools.faireUnBip();
						}
					}
					else if ( toucheClavier == ToucheClavier.ConnexionFin ){
						campJoueur = null;
						boucleParametresJeu = false;
						break;
					}
					else{
						minitelTools.faireUnBip();
					}
				}
			}
			minitelTools.setCursorBlinking(false);
			
			
			// Choix difficulté
			if ( campJoueur != null ){
				minitelTools.locate(13, 8);
				minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
				minitelTools.writeRepeteByte((byte) ' ', 7);
				
				minitelTools.locate(2, 11);
				minitelTools.write("Choisissez la difficulté:");
				minitelTools.locate(4, 12);
				minitelTools.setColorBackground(CodeCouleurDeFond.fondGris7);
				minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
				minitelTools.write(" 1 ");
				minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
				minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
				minitelTools.write(" Facile");
				minitelTools.locate(4, 13);
				minitelTools.setColorBackground(CodeCouleurDeFond.fondGris7);
				minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
				minitelTools.write(" 2 ");
				minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
				minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
				minitelTools.write(" Normale");
				
				minitelTools.locate(2, 15);
				minitelTools.write("Choix: ");
				minitelTools.setColorBackground(CodeCouleurDeFond.fondGris2);
				minitelTools.write(" . ");
				minitelTools.locate(13, 15);
				minitelTools.setColorBackground(CodeCouleurDeFond.fondGris6);
				minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
				minitelTools.write(" Envoi ");
				
				minitelTools.locate(21, 22);
				minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
				minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
				minitelTools.write("Annulation ");
				minitelTools.setColorBackground(CodeCouleurDeFond.fondGris6);
				minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
				minitelTools.write(" Annul.  ");
				minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
				minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
				
				minitelTools.locate(10, 15);
				minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
				minitelTools.setCursorBlinking(true);
				
				difficulte = null;
				while ( true ){
					ToucheClavier toucheClavier = minitelTools.readClavier();
					if ( toucheClavier != null ){
						
						if ( toucheClavier == ToucheClavier.Chiffre1 || toucheClavier == ToucheClavier.Chiffre2 ){
							if ( difficulte == null ){
								if ( toucheClavier == ToucheClavier.Chiffre1 ){
									difficulte = Difficulte.facile;
								}
								else{
									difficulte = Difficulte.normale;
								}
								minitelTools.write(toucheClavier.toString());
							}
							else{
								minitelTools.faireUnBip();
							}
						}
						else if ( toucheClavier == ToucheClavier.Correction ){
							if ( difficulte != null ){
								difficulte = null;
								minitelTools.locate(10, 15);
								minitelTools.write(".");
								minitelTools.locate(10, 15);
							}
							else{
								minitelTools.faireUnBip();
							}
						}
						else if ( toucheClavier == ToucheClavier.Annulation ){
							minitelTools.setCursorBlinking(false);
							minitelTools.locate(1, 11);
							minitelTools.clearLigneEnCours();
							minitelTools.locate(1, 12);
							minitelTools.clearLigneEnCours();
							minitelTools.locate(1, 13);
							minitelTools.clearLigneEnCours();
							minitelTools.locate(1, 14);
							minitelTools.clearLigneEnCours();
							minitelTools.locate(1, 15);
							minitelTools.clearLigneEnCours();
							
							break;
						}
						else if ( toucheClavier == ToucheClavier.Envoi || toucheClavier == ToucheClavier.Entree ){
							if ( difficulte != null ){
								boucleParametresJeu = false;
								affichageSuivant = Affichage.partie;
								break;
							}
							else{
								minitelTools.faireUnBip();
							}
						}
						else if ( toucheClavier == ToucheClavier.ConnexionFin ){
							boucleParametresJeu = false;
							break;
						}
						else{
							minitelTools.faireUnBip();
						}
					}
				}
				minitelTools.setCursorBlinking(false);
			}
			else{
				boucleParametresJeu = false;
			}
		}
		
		return affichageSuivant;
	}
	
	private Affichage partie() throws TransmissionException, ReceptionException{
		
		logger.debug("Nouvelle partie: " + campJoueur.name() + " - Difficulté: " + difficulte.name());
		
		minitelTools.clearAll();
		
		
		minitelTools.allerDansLigne00(1);
		lignePlateau(0, -1, true);
		minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
		minitelTools.write(" \n");
		minitelTools.locate(1, 1);
		lignePlateau(0, 8);
		minitelTools.locate(1, 2);
		lignePlateau(0, -1);
		
		int ligneEcran = 3;
		for ( int ligne = 7 ; ligne >= 1 ; ligne-- ){
			for ( int i = 0 ; i < 3 ; i++ ){
				minitelTools.locate(1, ligneEcran);
				if ( ligne == 6 || ligne == 4 || ligne == 2 ){
					if ( i == 1 ){
						lignePlateau(0, ligne);
					}
					else{
						lignePlateau(0, -1);
					}
				}
				else{
					if ( i == 1 ){
						lignePlateau(1, ligne);
					}
					else{
						lignePlateau(1, -1);
					}
				}
				ligneEcran++;
			}
		}
		
		minitelTools.locate(29, 1);
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris6);
		minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
		minitelTools.write(" ECHECS ");
		minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
		
		minitelTools.locate(26, 3);
		minitelTools.write(TypePieceJeu.Roi.toString() + " = Roi");
		minitelTools.locate(26, 4);
		minitelTools.write(TypePieceJeu.Dame.toString() + " = Dame");
		minitelTools.locate(26, 5);
		minitelTools.write(TypePieceJeu.Fou.toString() + " = Fou");
		minitelTools.locate(26, 6);
		minitelTools.write(TypePieceJeu.Cavalier.toString() + " = Cavalier");
		minitelTools.locate(26, 7);
		minitelTools.write(TypePieceJeu.Tour.toString() + " = Tour");
		minitelTools.locate(26, 8);
		minitelTools.write(TypePieceJeu.Pion.toString() + " = Pion");
		
		minitelTools.locate(25, 12);
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris3);
		minitelTools.write(" Précédents     ");
		minitelTools.locate(25, 13);
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris3);
		minitelTools.write("         coups: ");
		minitelTools.locate(25, 14);
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris3);
		minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
		minitelTools.write("   Vous:");
		minitelTools.writeRepeteByte((byte) ' ', 7);
		minitelTools.locate(25, 15);
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris3);
		minitelTools.write(" Ennemi:");
		minitelTools.writeRepeteByte((byte) ' ', 7);
		
		minitelTools.locate(27, 22);
		minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
		minitelTools.write("Menu ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris6);
		minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
		minitelTools.write(" Sommaire");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
		minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
		
		minitelTools.locate(26, 23);
		minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
		minitelTools.write("Quit. ");
		minitelTools.setColorBackground(CodeCouleurDeFond.fondGris6);
		minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
		minitelTools.write(" ConnFin ");
		
		
		// Initialisation du plateau
		// [X][Y] coordonnées des pièces de jeu sur le plateau
		plateau = new PieceJeu[8][8];
		
		// Pièces du joueur
		plateau[0][0] = new PieceJeu(TypePieceJeu.Tour, campJoueur);
		plateau[1][0] = new PieceJeu(TypePieceJeu.Cavalier, campJoueur);
		plateau[2][0] = new PieceJeu(TypePieceJeu.Fou, campJoueur);
		plateau[3][0] = new PieceJeu(TypePieceJeu.Dame, campJoueur);
		plateau[4][0] = new PieceJeu(TypePieceJeu.Roi, campJoueur);
		plateau[5][0] = new PieceJeu(TypePieceJeu.Fou, campJoueur);
		plateau[6][0] = new PieceJeu(TypePieceJeu.Cavalier, campJoueur);
		plateau[7][0] = new PieceJeu(TypePieceJeu.Tour, campJoueur);
		plateau[0][1] = new PieceJeu(TypePieceJeu.Pion, campJoueur);
		plateau[1][1] = new PieceJeu(TypePieceJeu.Pion, campJoueur);
		plateau[2][1] = new PieceJeu(TypePieceJeu.Pion, campJoueur);
		plateau[3][1] = new PieceJeu(TypePieceJeu.Pion, campJoueur);
		plateau[4][1] = new PieceJeu(TypePieceJeu.Pion, campJoueur);
		plateau[5][1] = new PieceJeu(TypePieceJeu.Pion, campJoueur);
		plateau[6][1] = new PieceJeu(TypePieceJeu.Pion, campJoueur);
		plateau[7][1] = new PieceJeu(TypePieceJeu.Pion, campJoueur);
		
		// Pièces adversaire
		Camp campAdversaire = null;
		if ( campJoueur == Camp.noir ){
			campAdversaire = Camp.blanc;
		}
		else{
			campAdversaire = Camp.noir;
		}
		plateau[0][7] = new PieceJeu(TypePieceJeu.Tour, campAdversaire);
		plateau[1][7] = new PieceJeu(TypePieceJeu.Cavalier, campAdversaire);
		plateau[2][7] = new PieceJeu(TypePieceJeu.Fou, campAdversaire);
		plateau[3][7] = new PieceJeu(TypePieceJeu.Dame, campAdversaire);
		plateau[4][7] = new PieceJeu(TypePieceJeu.Roi, campAdversaire);
		plateau[5][7] = new PieceJeu(TypePieceJeu.Fou, campAdversaire);
		plateau[6][7] = new PieceJeu(TypePieceJeu.Cavalier, campAdversaire);
		plateau[7][7] = new PieceJeu(TypePieceJeu.Tour, campAdversaire);
		plateau[0][6] = new PieceJeu(TypePieceJeu.Pion, campAdversaire);
		plateau[1][6] = new PieceJeu(TypePieceJeu.Pion, campAdversaire);
		plateau[2][6] = new PieceJeu(TypePieceJeu.Pion, campAdversaire);
		plateau[3][6] = new PieceJeu(TypePieceJeu.Pion, campAdversaire);
		plateau[4][6] = new PieceJeu(TypePieceJeu.Pion, campAdversaire);
		plateau[5][6] = new PieceJeu(TypePieceJeu.Pion, campAdversaire);
		plateau[6][6] = new PieceJeu(TypePieceJeu.Pion, campAdversaire);
		plateau[7][6] = new PieceJeu(TypePieceJeu.Pion, campAdversaire);
		
		// Affichage des pièces
		for ( int y = 7 ; y >= 0 ; y-- ){
			if ( y == 0 || y == 1 || y == 6 || y == 7 ){
				for ( int x = 0 ; x <= 7 ; x++ ){
					afficherPieceJeu(plateau[x][y], x, y);
				}
			}
		}
		
		
		// Lancement du jeu
		Affichage affichageSuivant = null;
		ParticipantJeu tour = ParticipantJeu.Joueur;
		String coordonnesEntreeJoueur;
		boolean messageErreurJoueur = false;
		PieceJeu pieceJeuADeplacer;
		int nbCoupsJoueur = 0;
		int nbCoupsEnnemi = 0;
		boolean continuerPartie = true;
		while ( continuerPartie ){
			minitelTools.locate(26, 18);
			minitelTools.clearLigneDuCurseurALaFinDeLaLigne();
			
			if ( tour == ParticipantJeu.Joueur ){
				minitelTools.locate(26, 18);
				minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
				minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
				minitelTools.write("A vous: ");
				minitelTools.setColorBackground(CodeCouleurDeFond.fondGris2);
				minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
				minitelTools.write(" .... ");
				minitelTools.locate(35, 18);
				minitelTools.setCursorBlinking(true);
				
				coordonnesEntreeJoueur = "";
				messageErreurJoueur = false;
				while ( true ){
					ToucheClavier toucheClavier = minitelTools.readClavier();
					if ( toucheClavier != null ){
						
						if ( messageErreurJoueur ){
							minitelTools.locate(26, 19);
							minitelTools.clearLigneDuCurseurALaFinDeLaLigne();
							minitelTools.locate(35 + coordonnesEntreeJoueur.length(), 18);
							messageErreurJoueur = false;
						}
						
						if ( toucheClavier.getTypeToucheClavier() == TypeToucheClavier.Caractere ){
							if ( coordonnesEntreeJoueur.length() < 4 ){
								coordonnesEntreeJoueur += toucheClavier.toString();
								minitelTools.write(toucheClavier.toString());
							}
							else{
								minitelTools.faireUnBip();
							}
						}
						else if ( toucheClavier == ToucheClavier.Correction ){
							if ( coordonnesEntreeJoueur.length() > 0 ){
								coordonnesEntreeJoueur = coordonnesEntreeJoueur.substring(0, coordonnesEntreeJoueur.length() - 1);
								minitelTools.locate(35 + coordonnesEntreeJoueur.length(), 18);
								minitelTools.write(".");
								minitelTools.locate(35 + coordonnesEntreeJoueur.length(), 18);
							}
							else{
								minitelTools.faireUnBip();
							}
						}
						else if ( toucheClavier == ToucheClavier.Annulation ){
							if ( coordonnesEntreeJoueur.length() > 0 ){
								coordonnesEntreeJoueur = "";
								minitelTools.locate(35, 18);
								minitelTools.write("....");
								minitelTools.locate(35, 18);
							}
							else{
								minitelTools.faireUnBip();
							}
						}
						else if ( toucheClavier == ToucheClavier.Envoi ){
							if ( coordonnesEntreeJoueur.length() == 4 ){
								String coordonnesPieceAPrendre = coordonnesEntreeJoueur.toLowerCase().substring(0, 2);
								String coordonnesEndroitOuPoser = coordonnesEntreeJoueur.toLowerCase().substring(2, 4);
								int[] coordonnesPlateauPieceAPrendre = this.getCoordonnesIntPlateau(coordonnesPieceAPrendre);
								int[] coordonnesPlateauEndroitOuPoser = this.getCoordonnesIntPlateau(coordonnesEndroitOuPoser);
								if ( coordonnesPlateauPieceAPrendre != null && coordonnesPlateauEndroitOuPoser != null ){
									if ( verifierRegles(coordonnesPlateauPieceAPrendre, coordonnesPlateauEndroitOuPoser) ){
										pieceJeuADeplacer = this.getPieceJeuPlateau(coordonnesPlateauPieceAPrendre[0], coordonnesPlateauPieceAPrendre[1]);
										
										minitelTools.setCursorBlinking(false);
										
										this.setPieceJeuPlateau(null, coordonnesPlateauPieceAPrendre[0], coordonnesPlateauPieceAPrendre[1]);
										this.afficherPieceJeu(null, coordonnesPlateauPieceAPrendre[0], coordonnesPlateauPieceAPrendre[1]);
										this.setPieceJeuPlateau(pieceJeuADeplacer, coordonnesPlateauEndroitOuPoser[0], coordonnesPlateauEndroitOuPoser[1]);
										this.afficherPieceJeu(pieceJeuADeplacer, coordonnesPlateauEndroitOuPoser[0], coordonnesPlateauEndroitOuPoser[1]);
										pieceJeuADeplacer.setDejaDeplace(true);
										
										nbCoupsJoueur++;
										minitelTools.locate(33, 14);
										minitelTools.setColorBackground(CodeCouleurDeFond.fondGris3);
										minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
										minitelTools.write(nbCoupsJoueur + "");
										minitelTools.locate(36, 14);
										minitelTools.setColorBackground(CodeCouleurDeFond.fondGris3);
										minitelTools.write(coordonnesEntreeJoueur.toLowerCase());
										
										logger.debug("Mouvement joueur: " + coordonnesEntreeJoueur.toLowerCase());
										
										// Vérification s'il y a un gagnant
										if ( verificationGagnant() == null ){
											// Aucun gagnant pour l'instant
											
											// Vérification si un pion est arrivé sur la dernière rangée chez l'ennnemi
											if ( pieceJeuADeplacer.getTypePieceJeu() == TypePieceJeu.Pion && coordonnesPlateauEndroitOuPoser[1] == 7 ){
												// Promotion pion
												minitelTools.locate(26, 18);
												minitelTools.clearLigneDuCurseurALaFinDeLaLigne();
												minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
												minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
												minitelTools.write("Promotion: ");
												minitelTools.setColorBackground(CodeCouleurDeFond.fondGris2);
												minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
												minitelTools.write(" . ");
												minitelTools.locate(38, 18);
												minitelTools.setCursorBlinking(true);
												
												String promotion = "";
												while ( true ){
													toucheClavier = minitelTools.readClavier();
													if ( toucheClavier != null ){
														
														if ( toucheClavier.getTypeToucheClavier() == TypeToucheClavier.Caractere ){
															if ( promotion.length() == 0 ){
																promotion = toucheClavier.toString();
																minitelTools.write(promotion);
															}
															else{
																minitelTools.faireUnBip();
															}
														}
														else if ( toucheClavier == ToucheClavier.Annulation || toucheClavier == ToucheClavier.Correction ){
															if ( promotion.length() > 0 ){
																promotion = "";
																minitelTools.locate(38, 18);
																minitelTools.write(".");
																minitelTools.locate(38, 18);
															}
															else{
																minitelTools.faireUnBip();
															}
														}
														else if ( toucheClavier == ToucheClavier.Envoi ){
															if ( promotion.toUpperCase().equals(TypePieceJeu.Roi.toString()) || promotion.toUpperCase().equals(TypePieceJeu.Dame.toString()) ||
																	promotion.toUpperCase().equals(TypePieceJeu.Fou.toString()) || promotion.toUpperCase().equals(TypePieceJeu.Cavalier.toString()) ||
																	promotion.toUpperCase().equals(TypePieceJeu.Tour.toString()) ){
																TypePieceJeu promotionPieceJeu = null;
																if ( promotion.toUpperCase().equals(TypePieceJeu.Roi.toString()) ){
																	promotionPieceJeu = TypePieceJeu.Roi;
																}
																if ( promotion.toUpperCase().equals(TypePieceJeu.Dame.toString()) ){
																	promotionPieceJeu = TypePieceJeu.Dame;
																}
																if ( promotion.toUpperCase().equals(TypePieceJeu.Fou.toString()) ){
																	promotionPieceJeu = TypePieceJeu.Fou;
																}
																if ( promotion.toUpperCase().equals(TypePieceJeu.Cavalier.toString()) ){
																	promotionPieceJeu = TypePieceJeu.Cavalier;
																}
																if ( promotion.toUpperCase().equals(TypePieceJeu.Tour.toString()) ){
																	promotionPieceJeu = TypePieceJeu.Tour;
																}
																if ( promotionPieceJeu != null ){
																	pieceJeuADeplacer.setTypePieceJeu(promotionPieceJeu);
																	this.afficherPieceJeu(pieceJeuADeplacer, coordonnesPlateauEndroitOuPoser[0], coordonnesPlateauEndroitOuPoser[1]);
																	
																	logger.debug("Promotion pion " + coordonnesPlateauEndroitOuPoser[0] + "" + coordonnesPlateauEndroitOuPoser[1] + ": " + promotionPieceJeu.name());
																}
																
																break;
															}
															else{
																minitelTools.faireUnBip();
																minitelTools.locate(38, 18);
																minitelTools.write(".");
																minitelTools.locate(38, 18);
															}
														}
														else if ( toucheClavier == ToucheClavier.Sommaire ){
															tour = null;
															affichageSuivant = Affichage.accueil;
															continuerPartie = false;
															break;
														}
														else if ( toucheClavier == ToucheClavier.ConnexionFin ){
															tour = null;
															continuerPartie = false;
															break;
														}
														else{
															minitelTools.faireUnBip();
														}
													}
												}
												minitelTools.locate(26, 18);
												minitelTools.clearLigneDuCurseurALaFinDeLaLigne();
												
											}
											
											// Tour suivant
											tour = ParticipantJeu.Ennemi;
										}
										else{
											// Il y a un gagnant
											// Jeu terminé
											tour = null;
										}
										break;
									}
									else{
										this.messageErreurJoueur("Interdit");
										messageErreurJoueur = true;
										coordonnesEntreeJoueur = "";
									}
								}
								else{
									this.messageErreurJoueur("Incorrect");
									messageErreurJoueur = true;
									coordonnesEntreeJoueur = "";
								}
							}
							else{
								minitelTools.faireUnBip();
							}
						}
						else if ( toucheClavier == ToucheClavier.Sommaire ){
							tour = null;
							affichageSuivant = Affichage.accueil;
							continuerPartie = false;
							break;
						}
						else if ( toucheClavier == ToucheClavier.ConnexionFin ){
							tour = null;
							continuerPartie = false;
							break;
						}
						else{
							minitelTools.faireUnBip();
						}
					}
				}
				minitelTools.setCursorBlinking(false);
			}
			else if ( tour == ParticipantJeu.Ennemi ){
				minitelTools.locate(26, 18);
				minitelTools.clearLigneDuCurseurALaFinDeLaLigne();
				minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
				minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
				minitelTools.write("Réfléchi ...");
				
				String mouvementEnnemi = this.effectuerTourEnnemi();
				
				nbCoupsEnnemi++;
				minitelTools.locate(33, 15);
				minitelTools.setColorBackground(CodeCouleurDeFond.fondGris3);
				minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
				minitelTools.write(nbCoupsEnnemi + "");
				minitelTools.locate(36, 15);
				minitelTools.setColorBackground(CodeCouleurDeFond.fondGris3);
				minitelTools.write(mouvementEnnemi);
				
				// Vérification s'il y a un gagnant
				if ( verificationGagnant() == null ){
					// Aucun gagnant pour l'instant
					// Tour suivant
					tour = ParticipantJeu.Joueur;
				}
				else{
					// Il y a un gagnant
					// Jeu terminé
					tour = null;
				}
			}
			else{
				// Ce n'est plus le tour de personne, le jeu est fini
				ParticipantJeu gagnant = verificationGagnant();
				
				minitelTools.locate(26, 18);
				minitelTools.clearLigneDuCurseurALaFinDeLaLigne();
				minitelTools.setColorBackground(CodeCouleurDeFond.fondNoir);
				minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
				minitelTools.locate(26, 18);
				if ( gagnant == ParticipantJeu.Joueur ){
					minitelTools.write("GAGNE !");
				}
				else{
					minitelTools.write("Perdu ?..");
				}
				
				while ( true ){
					ToucheClavier toucheClavier = minitelTools.readClavier();
					if ( toucheClavier != null ){
						if ( toucheClavier == ToucheClavier.Sommaire ){
							affichageSuivant = Affichage.accueil;
							break;
						}
						else if ( toucheClavier == ToucheClavier.ConnexionFin ){
							break;
						}
						else{
							minitelTools.faireUnBip();
						}
					}
				}
				break;
			}
		}
		
		return affichageSuivant;
	}
	
	private String effectuerTourEnnemi() throws TransmissionException{
		String mouvementEnnemi = "";
		
		Camp campEnnemi = null;
		if ( campJoueur == Camp.noir ){
			campEnnemi = Camp.blanc;
		}
		else{
			campEnnemi = Camp.noir;
		}
		
		PieceJeu plateauEnnemi[][] = this.retournerPlateau(plateau);
		List<int[]> emplacementPiecesEnnemi = new ArrayList<int[]>();
		for ( int x = 0 ; x <= 7 ; x++ ){
			for ( int y = 0 ; y <= 7 ; y++ ){
				if ( plateauEnnemi[x][y] != null ){
					if ( plateauEnnemi[x][y].getCamp() == campEnnemi ){
						emplacementPiecesEnnemi.add(new int[]{ x, y });
					}
				}
			}
		}
		
		List<int[]> destinationsPossiblesParPiece;
		List<int[]> mouvementsPossibles = new ArrayList<int[]>();
		for ( int[] coordonnesPieceEnnemi : emplacementPiecesEnnemi ){
			destinationsPossiblesParPiece = this.destinationsPossiblesPieceJeu(plateauEnnemi, campEnnemi, coordonnesPieceEnnemi[0], coordonnesPieceEnnemi[1]);
			if ( destinationsPossiblesParPiece.size() > 0 ){
				for ( int[] destination : destinationsPossiblesParPiece ){
					mouvementsPossibles.add(new int[]{ coordonnesPieceEnnemi[0], coordonnesPieceEnnemi[1], destination[0], destination[1]});
				}
			}
		}
		
		int[] mouvement = null;
		if ( difficulte != Difficulte.facile ){
			final int nbMaxIterations;
			if ( difficulte == Difficulte.normale ){
				nbMaxIterations = 2;
			}
			else{
				nbMaxIterations = 0;
			}
			
			List<Object[]> reussitesPossibles = new ArrayList<Object[]>();
			
			// Recherche des réussites possibles de l'ennemi en mono-thread
			/*for ( int[] mouvementPossible : mouvementsPossibles ){
				
				// Application du mouvement dans un nouveau plateau
				PieceJeu[][] plateauApresTourEnnemi = new PieceJeu[8][8];
				for ( int x = 0 ; x <= 7 ; x++ ){
					for ( int y = 0 ; y <= 7 ; y++ ){
						if ( plateau[x][y] != null ){
							plateauApresTourEnnemi[x][y] = plateau[x][y].clone();
						}
					}
				}
				
				PieceJeu pieceJeuADeplacer = plateauApresTourEnnemi[7-mouvementPossible[0]][7-mouvementPossible[1]];
				pieceJeuADeplacer.setDejaDeplace(true);	
				plateauApresTourEnnemi[7-mouvementPossible[0]][7-mouvementPossible[1]] = null;
				plateauApresTourEnnemi[7-mouvementPossible[2]][7-mouvementPossible[3]] = pieceJeuADeplacer;
				
				// Vérification si après un mouvement possible, l'ennemi gagne
				if ( verificationGagnant(plateauApresTourEnnemi) == ParticipantJeu.Ennemi ){
					// Oui
					// Arrêt des recherches
					//mouvement = mouvementPossible;
					reussitesPossibles.add(new Object[]{ mouvementPossible, 0 });
				}
				else{
					// Pas encore
					// Simulation d'un mouvement de joueur et d'un mouvement de l'ennemi
					this.rechercherReussiteEnnemi(plateauApresTourEnnemi, mouvementPossible, reussitesPossibles, nbMaxIterations, 0);
				}
			}*/
			
			// Recherche des réussites possibles de l'ennemi en multi-thread
			Thread[] threads = new Thread[mouvementsPossibles.size()];
			Semaphore mutexReussitesPossibles = new Semaphore(1);
			for ( int i = 0 ; i < mouvementsPossibles.size() ; i++ ){
				final int[] mouvementPossible = mouvementsPossibles.get(i);
				threads[i] = new Thread("Thread" + i){
					public void run(){
						// Application du mouvement dans un nouveau plateau
						PieceJeu[][] plateauApresTourEnnemi = new PieceJeu[8][8];
						for ( int x = 0 ; x <= 7 ; x++ ){
							for ( int y = 0 ; y <= 7 ; y++ ){
								if ( plateau[x][y] != null ){
									plateauApresTourEnnemi[x][y] = plateau[x][y].clone();
								}
							}
						}
						
						PieceJeu pieceJeuADeplacer = plateauApresTourEnnemi[7-mouvementPossible[0]][7-mouvementPossible[1]];
						pieceJeuADeplacer.setDejaDeplace(true);	
						plateauApresTourEnnemi[7-mouvementPossible[0]][7-mouvementPossible[1]] = null;
						plateauApresTourEnnemi[7-mouvementPossible[2]][7-mouvementPossible[3]] = pieceJeuADeplacer;
						
						// Vérification si après un mouvement possible, l'ennemi gagne
						if ( verificationGagnant(plateauApresTourEnnemi) == ParticipantJeu.Ennemi ){
							// Oui
							// Arrêt des recherches
							try{
								mutexReussitesPossibles.acquire();
								reussitesPossibles.add(new Object[]{ mouvementPossible, 0 });
								mutexReussitesPossibles.release();
							}
							catch(Exception e){
								logger.debug(e);
							}
						}
						else{
							// Pas encore
							// Simulation d'un mouvement de joueur et d'un mouvement de l'ennemi
							rechercherReussiteEnnemi(plateauApresTourEnnemi, mouvementPossible, reussitesPossibles, mutexReussitesPossibles, nbMaxIterations, 0);
						}
						logger.debug(this.getName() + " terminé");
					}
				};
				threads[i].start();
				logger.debug("Démarrage " + threads[i].getName() + " de recherche de reussite possible de l'ennemi");
			}
			
			// Attente que tous les threads soit terminés
			boolean tousLesThreadsTermines = false;
			while ( !tousLesThreadsTermines ){
				tousLesThreadsTermines = true;
				for ( int i = 0 ; i < threads.length ; i++ ){
					if ( threads[i].isAlive() ){
						tousLesThreadsTermines = false;
					}
				}
				try{
					Thread.sleep(100);
				}
				catch(Exception e){
					logger.debug(e);
				}
			}
			
			logger.debug("Réussites possibles ennemi: " + reussitesPossibles.size());
			if ( reussitesPossibles.size() > 0 ){
				// Tri par ordre croissant du nombre de coups
				Collections.sort(reussitesPossibles, new Comparator<Object[]>() {
					@Override
					public int compare(Object[] reussite1, Object[] reussite2) {
						return Integer.compare( ((int)reussite1[1]) ,  ((int)reussite2[1]) );
					}
				});
				
				Object[] meilleureReussite = reussitesPossibles.get(0);
				mouvement = (int[]) meilleureReussite[0];
				logger.debug("Choix mouvement " + mouvement[0] + "" + mouvement[1] + "" + mouvement[2] + "" + mouvement[3] + " avec réussite possible en " + ((int) meilleureReussite[1] + 1) + " coups");
			}
			else{
				logger.debug("Aucune réussite => choix random");
			}
		}
		else{
			// Difficulté facile
			// On laisse faire le random
		}
		if ( mouvement == null && mouvementsPossibles.size() > 0 ){
			// Choix aléatoire d'un mouvement
			Random rand = new Random();
			int choix = rand.nextInt(mouvementsPossibles.size());
			mouvement = mouvementsPossibles.get(choix);
		}
		
		if ( mouvement != null ){
			// Application du mouvement sur le plateau du joueur
			PieceJeu pieceJeuADeplacer = plateau[7-mouvement[0]][7-mouvement[1]];
			pieceJeuADeplacer.setDejaDeplace(true);
					
			this.setPieceJeuPlateau(null, 7-mouvement[0], 7-mouvement[1]);
			this.afficherPieceJeu(null, 7-mouvement[0], 7-mouvement[1]);
			this.setPieceJeuPlateau(pieceJeuADeplacer, 7-mouvement[2], 7-mouvement[3]);
			this.afficherPieceJeu(pieceJeuADeplacer, 7-mouvement[2], 7-mouvement[3]);
			
			switch(7-mouvement[0]){
				case 0:
					mouvementEnnemi = "a";
					break;
				case 1:
					mouvementEnnemi = "b";
					break;
				case 2:
					mouvementEnnemi = "c";
					break;
				case 3:
					mouvementEnnemi = "d";
					break;
				case 4:
					mouvementEnnemi = "e";
					break;
				case 5:
					mouvementEnnemi = "f";
					break;
				case 6:
					mouvementEnnemi = "g";
					break;
				case 7:
					mouvementEnnemi = "h";
					break;
			}
			mouvementEnnemi += "" + (7-mouvement[1]+1);
			switch(7-mouvement[2]){
				case 0:
					mouvementEnnemi += "a";
					break;
				case 1:
					mouvementEnnemi += "b";
					break;
				case 2:
					mouvementEnnemi += "c";
					break;
				case 3:
					mouvementEnnemi += "d";
					break;
				case 4:
					mouvementEnnemi += "e";
					break;
				case 5:
					mouvementEnnemi += "f";
					break;
				case 6:
					mouvementEnnemi += "g";
					break;
				case 7:
					mouvementEnnemi += "h";
					break;
			}
			mouvementEnnemi += "" + (7-mouvement[3]+1);
			logger.debug("Mouvement ennemi: " + mouvementEnnemi);
		}
		
		return mouvementEnnemi;
	}
	
	private void rechercherReussiteEnnemi(PieceJeu plateauJoueur[][], int[] premierMouvementPossible, List<Object[]> reussitesPossibles, Semaphore mutexReussitesPossibles, int maxIterations, int iterationEnCours){
		iterationEnCours++;
		if ( iterationEnCours <= maxIterations ){
			
			Camp campEnnemi = null;
			if ( campJoueur == Camp.noir ){
				campEnnemi = Camp.blanc;
			}
			else{
				campEnnemi = Camp.noir;
			}
			
			List<int[]> emplacementsPiecesJoueur = new ArrayList<int[]>();
			for ( int x = 0 ; x <= 7 ; x++ ){
				for ( int y = 0 ; y <= 7 ; y++ ){
					if ( plateauJoueur[x][y] != null ){
						if ( plateauJoueur[x][y].getCamp() == campJoueur ){
							emplacementsPiecesJoueur.add(new int[]{ x, y });
						}
					}
				}
			}
			
			// Recherche des différents mouvements possible du joueur
			List<int[]> destinationsPossiblesParPiece;
			List<int[]> mouvementsPossibles = new ArrayList<int[]>();
			for ( int[] coordonnesPieceJoueur : emplacementsPiecesJoueur ){
				destinationsPossiblesParPiece = this.destinationsPossiblesPieceJeu(plateauJoueur, campJoueur, coordonnesPieceJoueur[0], coordonnesPieceJoueur[1]);
				if ( destinationsPossiblesParPiece.size() > 0 ){
					for ( int[] destination : destinationsPossiblesParPiece ){
						mouvementsPossibles.add(new int[]{ coordonnesPieceJoueur[0], coordonnesPieceJoueur[1], destination[0], destination[1]});
					}
				}
			}
			
			// Application de chaque mouvement dans un nouveau plateau
			for ( int[] mouvementPossible : mouvementsPossibles ){
				// Application du mouvement dans un nouveau plateau
				PieceJeu[][] plateauApresTourJoueur = new PieceJeu[8][8];
				for ( int x = 0 ; x <= 7 ; x++ ){
					for ( int y = 0 ; y <= 7 ; y++ ){
						if ( plateauJoueur[x][y] != null ){
							plateauApresTourJoueur[x][y] = plateauJoueur[x][y].clone();
						}
					}
				}
				
				PieceJeu pieceJeuADeplacer = plateauApresTourJoueur[mouvementPossible[0]][mouvementPossible[1]];
				pieceJeuADeplacer.setDejaDeplace(true);	
				plateauApresTourJoueur[mouvementPossible[0]][mouvementPossible[1]] = null;
				plateauApresTourJoueur[mouvementPossible[2]][mouvementPossible[3]] = pieceJeuADeplacer;
				
				if ( verificationGagnant(plateauApresTourJoueur) != ParticipantJeu.Joueur ){
					// Si le joueur n'a pas gagné, on continu
					
					// Recherche des mouvements possibles de l'ennemi après le mouvement du joueur
					PieceJeu plateauEnnemiApresTourJoueur[][] = this.retournerPlateau(plateauApresTourJoueur);
					List<int[]> emplacementPiecesEnnemi = new ArrayList<int[]>();
					for ( int x = 0 ; x <= 7 ; x++ ){
						for ( int y = 0 ; y <= 7 ; y++ ){
							if ( plateauEnnemiApresTourJoueur[x][y] != null ){
								if ( plateauEnnemiApresTourJoueur[x][y].getCamp() == campEnnemi ){
									emplacementPiecesEnnemi.add(new int[]{ x, y });
								}
							}
						}
					}
					
					List<int[]> destinationsPossiblesEnnemiParPiece;
					List<int[]> mouvementsPossiblesEnnemi = new ArrayList<int[]>();
					for ( int[] coordonnesPieceEnnemi : emplacementPiecesEnnemi ){
						destinationsPossiblesEnnemiParPiece = this.destinationsPossiblesPieceJeu(plateauEnnemiApresTourJoueur, campEnnemi, coordonnesPieceEnnemi[0], coordonnesPieceEnnemi[1]);
						if ( destinationsPossiblesEnnemiParPiece.size() > 0 ){
							for ( int[] destination : destinationsPossiblesEnnemiParPiece ){
								mouvementsPossiblesEnnemi.add(new int[]{ coordonnesPieceEnnemi[0], coordonnesPieceEnnemi[1], destination[0], destination[1]});
							}
						}
					}
					
					for ( int[] mouvementPossibleEnnemi : mouvementsPossiblesEnnemi ){
						
						// Application du mouvement dans un nouveau plateau
						PieceJeu[][] plateauApresTourEnnemi = new PieceJeu[8][8];
						for ( int x = 0 ; x <= 7 ; x++ ){
							for ( int y = 0 ; y <= 7 ; y++ ){
								if ( plateauApresTourJoueur[x][y] != null ){
									plateauApresTourEnnemi[x][y] = plateauApresTourJoueur[x][y].clone();
								}
							}
						}
						
						PieceJeu pieceJeuEnnemiADeplacer = plateauApresTourEnnemi[7-mouvementPossibleEnnemi[0]][7-mouvementPossibleEnnemi[1]];
						pieceJeuEnnemiADeplacer.setDejaDeplace(true);	
						plateauApresTourEnnemi[7-mouvementPossibleEnnemi[0]][7-mouvementPossibleEnnemi[1]] = null;
						plateauApresTourEnnemi[7-mouvementPossibleEnnemi[2]][7-mouvementPossibleEnnemi[3]] = pieceJeuEnnemiADeplacer;
						
						// Vérification si après un mouvement possible, l'ennemi gagne
						if ( verificationGagnant(plateauApresTourEnnemi) == ParticipantJeu.Ennemi ){
							// Oui
							// Arrêt des recherches
							//mouvement = mouvementPossibleEnnemi;
							
							boolean reussitePossibleExisteDeja = false;
							for ( Object[] reussitePossible : reussitesPossibles ){
								int[] premierMouvementReussite = (int[]) reussitePossible[0];
								int nbCoups = (int) reussitePossible[1];
								if ( premierMouvementReussite[0] == premierMouvementPossible[0] && premierMouvementReussite[1] == premierMouvementPossible[1] && nbCoups == iterationEnCours ){
									reussitePossibleExisteDeja = true;
									break;
								}
							}
							if ( !reussitePossibleExisteDeja ){
								try{
									mutexReussitesPossibles.acquire();
									logger.debug("Réussite possible ennemi en " + iterationEnCours + " coups en commençant par " + (7-premierMouvementPossible[0]) + "" + (7-premierMouvementPossible[1]) + "" + premierMouvementPossible[2] + "" + premierMouvementPossible[3] + "");
									reussitesPossibles.add(new Object[]{ premierMouvementPossible, iterationEnCours });
									mutexReussitesPossibles.release();
								}
								catch(Exception e){
									logger.debug(e);
								}
							}
						}
						else{
							// Pas encore
							// Simulation d'un mouvement de joueur et d'un mouvement de l'ennemi
							this.rechercherReussiteEnnemi(plateauApresTourEnnemi, premierMouvementPossible, reussitesPossibles, mutexReussitesPossibles, maxIterations, iterationEnCours);
						}
					}
				}
			}
		}
	}
	
	private PieceJeu[][] retournerPlateau(PieceJeu[][] plateau){
		PieceJeu plateauRetourne[][] = new PieceJeu[8][8];
		for ( int x = 0 ; x <= 7 ; x++ ){
			for ( int y = 0 ; y <= 7 ; y++ ){
				plateauRetourne[x][y] = plateau[7-x][7-y];
			}
		}
		return plateauRetourne;
	}
	
	private boolean verifierRegles(int[] coordonnesPlateauPieceAPrendre, int[] coordonnesPlateauEndroitOuPoser){
		boolean result = false;
		PieceJeu pieceJeuADeplacer = this.getPieceJeuPlateau(coordonnesPlateauPieceAPrendre[0], coordonnesPlateauPieceAPrendre[1]);
		if ( pieceJeuADeplacer != null ){
			if ( pieceJeuADeplacer.getCamp() == campJoueur ){
				PieceJeu pieceJeuALemplacementDeDestination = this.getPieceJeuPlateau(coordonnesPlateauEndroitOuPoser[0], coordonnesPlateauEndroitOuPoser[1]);
				boolean destinationOk = true;
				if ( pieceJeuALemplacementDeDestination != null ){
					if ( pieceJeuALemplacementDeDestination.getCamp() == campJoueur ){
						destinationOk = false;
					}
				}
				if ( destinationOk ){
					
					List<int[]> destinationsPossibles = this.destinationsPossiblesPieceJeu(plateau, campJoueur, coordonnesPlateauPieceAPrendre[0], coordonnesPlateauPieceAPrendre[1]);
					for ( int[] destination : destinationsPossibles ){
						if ( destination[0] == coordonnesPlateauEndroitOuPoser[0] && destination[1] == coordonnesPlateauEndroitOuPoser[1] ){
							result = true;
							break;
						}
					}
					
				}
			}
		}
		return result;
	}
	
	private List<int[]> destinationsPossiblesPieceJeu(PieceJeu[][] plateau, Camp campJoueur, int xPieceJeu, int yPieceJeu){
		List<int[]> destinationsPossibles = new ArrayList<int[]>();
		
		PieceJeu pieceJeuADeplacer = plateau[xPieceJeu][yPieceJeu];
		
		// Vérification des régles de déplacement en fonction du type de la pièce
		PieceJeu autrePieceJeu;
		
		if ( pieceJeuADeplacer.getTypePieceJeu() == TypePieceJeu.Pion ){
			if ( yPieceJeu+1 <= 7 ){
				// x y+1 / x y+2
				if ( plateau[xPieceJeu][yPieceJeu+1] == null ){
					destinationsPossibles.add(new int[]{xPieceJeu, yPieceJeu+1});
					if ( yPieceJeu+2 <= 7 ){
						if ( plateau[xPieceJeu][yPieceJeu+2] == null && !pieceJeuADeplacer.isDejaDeplace() ){
							destinationsPossibles.add(new int[]{xPieceJeu, yPieceJeu+2});
						}
					}
				}
				// x-1 y+1 (si pièce ennemi)
				if ( xPieceJeu > 0 ){
					autrePieceJeu = plateau[xPieceJeu-1][yPieceJeu+1];
					if ( autrePieceJeu != null ){
						if ( autrePieceJeu.getCamp() != campJoueur ){
							destinationsPossibles.add(new int[]{xPieceJeu-1, yPieceJeu+1});
						}
					}
				}
				// x+1 y+1 (si pièce ennemi)
				if ( xPieceJeu < 7 ){
					autrePieceJeu = plateau[xPieceJeu+1][yPieceJeu+1];
					if ( autrePieceJeu != null ){
						if ( autrePieceJeu.getCamp() != campJoueur ){
							destinationsPossibles.add(new int[]{xPieceJeu+1, yPieceJeu+1});
						}
					}
				}
			}
		}
		if ( pieceJeuADeplacer.getTypePieceJeu() == TypePieceJeu.Tour || pieceJeuADeplacer.getTypePieceJeu() == TypePieceJeu.Dame ){
			for ( int y = yPieceJeu+1 ; y <= 7 ; y++ ){
				autrePieceJeu = plateau[xPieceJeu][y];
				if ( autrePieceJeu == null ){
					destinationsPossibles.add(new int[]{xPieceJeu, y});
				}
				else{
					if ( autrePieceJeu.getCamp() != campJoueur ){
						destinationsPossibles.add(new int[]{xPieceJeu, y});
					}
					break;
				}
			}
			for ( int y = yPieceJeu-1 ; y >= 0 ; y-- ){
				autrePieceJeu = plateau[xPieceJeu][y];
				if ( autrePieceJeu == null ){
					destinationsPossibles.add(new int[]{xPieceJeu, y});
				}
				else{
					if ( autrePieceJeu.getCamp() != campJoueur ){
						destinationsPossibles.add(new int[]{xPieceJeu, y});
					}
					break;
				}
			}
			for ( int x = xPieceJeu+1 ; x <= 7 ; x++ ){
				autrePieceJeu = plateau[x][yPieceJeu];
				if ( autrePieceJeu == null ){
					destinationsPossibles.add(new int[]{x, yPieceJeu});
				}
				else{
					if ( autrePieceJeu.getCamp() != campJoueur ){
						destinationsPossibles.add(new int[]{x, yPieceJeu});
					}
					break;
				}
			}
			for ( int x = xPieceJeu-1 ; x >= 0 ; x-- ){
				autrePieceJeu = plateau[x][yPieceJeu];
				if ( autrePieceJeu == null ){
					destinationsPossibles.add(new int[]{x, yPieceJeu});
				}
				else{
					if ( autrePieceJeu.getCamp() != campJoueur ){
						destinationsPossibles.add(new int[]{x, yPieceJeu});
					}
					break;
				}
			}
		}
		if ( pieceJeuADeplacer.getTypePieceJeu() == TypePieceJeu.Fou || pieceJeuADeplacer.getTypePieceJeu() == TypePieceJeu.Dame){
			for ( int xy = 1 ; xy <= 7 ; xy++ ){
				if ( xPieceJeu+xy <= 7 && yPieceJeu+xy <= 7 ){
					autrePieceJeu = plateau[xPieceJeu+xy][yPieceJeu+xy];
					if ( autrePieceJeu == null ){
						destinationsPossibles.add(new int[]{xPieceJeu+xy, yPieceJeu+xy});
					}
					else{
						if ( autrePieceJeu.getCamp() != campJoueur ){
							destinationsPossibles.add(new int[]{xPieceJeu+xy, yPieceJeu+xy});
						}
						break;
					}
				}
				else{
					break;
				}
			}
			for ( int xy = 1 ; xy <= 7 ; xy++ ){
				if ( xPieceJeu+xy <= 7 && yPieceJeu-xy >= 0 ){
					autrePieceJeu = plateau[xPieceJeu+xy][yPieceJeu-xy];
					if ( autrePieceJeu == null ){
						destinationsPossibles.add(new int[]{xPieceJeu+xy, yPieceJeu-xy});
					}
					else{
						if ( autrePieceJeu.getCamp() != campJoueur ){
							destinationsPossibles.add(new int[]{xPieceJeu+xy, yPieceJeu-xy});
						}
						break;
					}
				}
				else{
					break;
				}
			}
			for ( int xy = 1 ; xy <= 7 ; xy++ ){
				if ( xPieceJeu-xy >= 0 && yPieceJeu+xy <= 7 ){
					autrePieceJeu = plateau[xPieceJeu-xy][yPieceJeu+xy];
					if ( autrePieceJeu == null ){
						destinationsPossibles.add(new int[]{xPieceJeu-xy, yPieceJeu+xy});
					}
					else{
						if ( autrePieceJeu.getCamp() != campJoueur ){
							destinationsPossibles.add(new int[]{xPieceJeu-xy, yPieceJeu+xy});
						}
						break;
					}
				}
				else{
					break;
				}
			}
			for ( int xy = 1 ; xy <= 7 ; xy++ ){
				if ( xPieceJeu-xy >= 0 && yPieceJeu-xy >= 0 ){
					autrePieceJeu = plateau[xPieceJeu-xy][yPieceJeu-xy];
					if ( autrePieceJeu == null ){
						destinationsPossibles.add(new int[]{xPieceJeu-xy, yPieceJeu-xy});
					}
					else{
						if ( autrePieceJeu.getCamp() != campJoueur ){
							destinationsPossibles.add(new int[]{xPieceJeu-xy, yPieceJeu-xy});
						}
						break;
					}
				}
				else{
					break;
				}
			}
		}
		if ( pieceJeuADeplacer.getTypePieceJeu() == TypePieceJeu.Roi ){
			if ( yPieceJeu+1 <= 7 ){
				if ( xPieceJeu-1 >= 0 ){
					autrePieceJeu = plateau[xPieceJeu-1][yPieceJeu+1];
					if ( autrePieceJeu == null ){
						destinationsPossibles.add(new int[]{xPieceJeu-1, yPieceJeu+1});
					}
					else if ( autrePieceJeu.getCamp() != campJoueur ){
						destinationsPossibles.add(new int[]{xPieceJeu-1, yPieceJeu+1});
					}
				}
				autrePieceJeu = plateau[xPieceJeu][yPieceJeu+1];
				if ( autrePieceJeu == null ){
					destinationsPossibles.add(new int[]{xPieceJeu, yPieceJeu+1});
				}
				else if ( autrePieceJeu.getCamp() != campJoueur ){
					destinationsPossibles.add(new int[]{xPieceJeu, yPieceJeu+1});
				}
				if ( xPieceJeu+1 <= 7 ){
					autrePieceJeu = plateau[xPieceJeu+1][yPieceJeu+1];
					if ( autrePieceJeu == null ){
						destinationsPossibles.add(new int[]{xPieceJeu+1, yPieceJeu+1});
					}
					else if ( autrePieceJeu.getCamp() != campJoueur ){
						destinationsPossibles.add(new int[]{xPieceJeu+1, yPieceJeu+1});
					}
				}
			}
			if ( yPieceJeu-1 >= 0 ){
				if ( xPieceJeu-1 >= 0 ){
					autrePieceJeu = plateau[xPieceJeu-1][yPieceJeu-1];
					if ( autrePieceJeu == null ){
						destinationsPossibles.add(new int[]{xPieceJeu-1, yPieceJeu-1});
					}
					else if ( autrePieceJeu.getCamp() != campJoueur ){
						destinationsPossibles.add(new int[]{xPieceJeu-1, yPieceJeu-1});
					}
				}
				autrePieceJeu = plateau[xPieceJeu][yPieceJeu-1];
				if ( autrePieceJeu == null ){
					destinationsPossibles.add(new int[]{xPieceJeu, yPieceJeu-1});
				}
				else if ( autrePieceJeu.getCamp() != campJoueur ){
					destinationsPossibles.add(new int[]{xPieceJeu, yPieceJeu-1});
				}
				if ( xPieceJeu+1 <= 7 ){
					autrePieceJeu = plateau[xPieceJeu+1][yPieceJeu-1];
					if ( autrePieceJeu == null ){
						destinationsPossibles.add(new int[]{xPieceJeu+1, yPieceJeu-1});
					}
					else if ( autrePieceJeu.getCamp() != campJoueur ){
						destinationsPossibles.add(new int[]{xPieceJeu+1, yPieceJeu-1});
					}
				}
			}
			if ( xPieceJeu-1 >= 0 ){
				autrePieceJeu = plateau[xPieceJeu-1][yPieceJeu];
				if ( autrePieceJeu == null ){
					destinationsPossibles.add(new int[]{xPieceJeu-1, yPieceJeu});
				}
				else if ( autrePieceJeu.getCamp() != campJoueur ){
					destinationsPossibles.add(new int[]{xPieceJeu-1, yPieceJeu});
				}
			}
			if ( xPieceJeu+1 <= 7 ){
				autrePieceJeu = plateau[xPieceJeu+1][yPieceJeu];
				if ( autrePieceJeu == null ){
					destinationsPossibles.add(new int[]{xPieceJeu+1, yPieceJeu});
				}
				else if ( autrePieceJeu.getCamp() != campJoueur ){
					destinationsPossibles.add(new int[]{xPieceJeu+1, yPieceJeu});
				}
			}
		}
		if ( pieceJeuADeplacer.getTypePieceJeu() == TypePieceJeu.Cavalier ){
			if ( xPieceJeu+1 <= 7 && yPieceJeu+2 <= 7 ){
				autrePieceJeu = plateau[xPieceJeu+1][yPieceJeu+2];
				if ( autrePieceJeu == null ){
					destinationsPossibles.add(new int[]{xPieceJeu+1, yPieceJeu+2});
				}
				else if ( autrePieceJeu.getCamp() != campJoueur ){
					destinationsPossibles.add(new int[]{xPieceJeu+1, yPieceJeu+2});
				}
			}
			if ( xPieceJeu+2 <= 7 && yPieceJeu+1 <= 7 ){
				autrePieceJeu = plateau[xPieceJeu+2][yPieceJeu+1];
				if ( autrePieceJeu == null ){
					destinationsPossibles.add(new int[]{xPieceJeu+2, yPieceJeu+1});
				}
				else if ( autrePieceJeu.getCamp() != campJoueur ){
					destinationsPossibles.add(new int[]{xPieceJeu+2, yPieceJeu+1});
				}
			}
			if ( xPieceJeu+2 <= 7 && yPieceJeu-1 >= 0 ){
				autrePieceJeu = plateau[xPieceJeu+2][yPieceJeu-1];
				if ( autrePieceJeu == null ){
					destinationsPossibles.add(new int[]{xPieceJeu+2, yPieceJeu-1});
				}
				else if ( autrePieceJeu.getCamp() != campJoueur ){
					destinationsPossibles.add(new int[]{xPieceJeu+2, yPieceJeu-1});
				}
			}
			if ( xPieceJeu+1 <= 7 && yPieceJeu-2 >= 0 ){
				autrePieceJeu = plateau[xPieceJeu+1][yPieceJeu-2];
				if ( autrePieceJeu == null ){
					destinationsPossibles.add(new int[]{xPieceJeu+1, yPieceJeu-2});
				}
				else if ( autrePieceJeu.getCamp() != campJoueur ){
					destinationsPossibles.add(new int[]{xPieceJeu+1, yPieceJeu-2});
				}
			}
			if ( xPieceJeu-1 >= 0 && yPieceJeu-2 >= 0 ){
				autrePieceJeu = plateau[xPieceJeu-1][yPieceJeu-2];
				if ( autrePieceJeu == null ){
					destinationsPossibles.add(new int[]{xPieceJeu-1, yPieceJeu-2});
				}
				else if ( autrePieceJeu.getCamp() != campJoueur ){
					destinationsPossibles.add(new int[]{xPieceJeu-1, yPieceJeu-2});
				}
			}
			if ( xPieceJeu-2 >= 0 && yPieceJeu-1 >= 0 ){
				autrePieceJeu = plateau[xPieceJeu-2][yPieceJeu-1];
				if ( autrePieceJeu == null ){
					destinationsPossibles.add(new int[]{xPieceJeu-2, yPieceJeu-1});
				}
				else if ( autrePieceJeu.getCamp() != campJoueur ){
					destinationsPossibles.add(new int[]{xPieceJeu-2, yPieceJeu-1});
				}
			}
			if ( xPieceJeu-2 >= 0 && yPieceJeu+1 <= 7 ){
				autrePieceJeu = plateau[xPieceJeu-2][yPieceJeu+1];
				if ( autrePieceJeu == null ){
					destinationsPossibles.add(new int[]{xPieceJeu-2, yPieceJeu+1});
				}
				else if ( autrePieceJeu.getCamp() != campJoueur ){
					destinationsPossibles.add(new int[]{xPieceJeu-2, yPieceJeu+1});
				}
			}
			if ( xPieceJeu-1 >= 0 && yPieceJeu+2 <= 7 ){
				autrePieceJeu = plateau[xPieceJeu-1][yPieceJeu+2];
				if ( autrePieceJeu == null ){
					destinationsPossibles.add(new int[]{xPieceJeu-1, yPieceJeu+2});
				}
				else if ( autrePieceJeu.getCamp() != campJoueur ){
					destinationsPossibles.add(new int[]{xPieceJeu-1, yPieceJeu+2});
				}
			}
		}
		
		return destinationsPossibles;
	}
	
	private ParticipantJeu verificationGagnant(){
		return this.verificationGagnant(plateau);
	}
	
	private ParticipantJeu verificationGagnant(PieceJeu[][] plateau){
		boolean presenceRoiBlanc = false;
		boolean presenceRoiNoir = false;
		PieceJeu pieceJeu;
		for ( int x = 0 ; x < 8 ; x++ ){
			for ( int y = 0 ; y < 8 ; y++ ){
				pieceJeu = plateau[x][y];
				if ( pieceJeu != null ){
					if ( pieceJeu.getTypePieceJeu() == TypePieceJeu.Roi ){
						if ( pieceJeu.getCamp() == Camp.noir ){
							presenceRoiNoir = true;
						}
						else{
							presenceRoiBlanc = true;
						}
					}
				}
			}
		}
		ParticipantJeu gagnant = null;
		if ( !presenceRoiBlanc ){
			if ( campJoueur == Camp.blanc ){
				gagnant = ParticipantJeu.Ennemi;
			}
			else{
				gagnant = ParticipantJeu.Joueur;
			}
		}
		if ( !presenceRoiNoir ){
			if ( campJoueur == Camp.noir ){
				gagnant = ParticipantJeu.Ennemi;
			}
			else{
				gagnant = ParticipantJeu.Joueur;
			}
		}
		return gagnant;
	}
	
	private void messageErreurJoueur(String message) throws TransmissionException{
		minitelTools.locate(26, 19);
		minitelTools.write(message);
		minitelTools.locate(35, 18);
		minitelTools.write("....");
		minitelTools.locate(35, 18);
	}
	
	private void afficherPieceJeu(PieceJeu pieceJeu, int x, int y) throws TransmissionException{
		if ( x >= 0 && x <= 7 && y >= 0 && y <= 7 ){
			int[] coordonnesEcran = this.getCoordonnesEcranPlateau(x, y);
			if ( coordonnesEcran != null ){
				minitelTools.locate(coordonnesEcran[0], coordonnesEcran[1]);
				minitelTools.setColorBackground(this.getCouleurDeFondCoordonnesPlateau(x, y));
				if ( pieceJeu != null ){
					if ( pieceJeu.getCamp() == Camp.noir ){
						minitelTools.setColorText(CodeCouleurCaractere.caractereNoir);
					}
					else{
						minitelTools.setColorText(CodeCouleurCaractere.caractereBlanc);
					}
					minitelTools.write(" " + pieceJeu.getTypePieceJeu().toString());
				}
				else{
					minitelTools.write("  ");
				}
			}
		}
	}
	
	private CodeCouleurDeFond getCouleurDeFondCoordonnesPlateau(int x, int y){
		CodeCouleurDeFond couleur = null;
		if ( x >= 0 && x <= 7 && y >= 0 && y <= 7 ){
			if ( x == 0 || x == 2 || x == 4 || x == 6 ){
				if ( y == 0 || y == 2 || y == 4 || y == 6 ){
					return CodeCouleurDeFond.fondGris2;
				}
				else{
					return CodeCouleurDeFond.fondGris4;
				}
			}
			else{
				if ( y == 0 || y == 2 || y == 4 || y == 6 ){
					return CodeCouleurDeFond.fondGris4;
				}
				else{
					return CodeCouleurDeFond.fondGris2;
				}
			}
		}
		return couleur;
	}
	
	private int[] getCoordonnesEcranPlateau(int x, int y){
		int[] coordonnes = null;
		if ( x >= 0 && x <= 7 && y >= 0 && y <= 7 ){
			coordonnes = new int[]{ x * 3 + 1 , 22 - y * 3 };
		}
		return coordonnes;
	}
	
	/*private void setPieceJeuPlateau(PieceJeu pieceJeu, String coordonnes){
		int[] coordonnesInt = this.getCoordonnesIntPlateau(coordonnes);
		if ( coordonnesInt != null ){
			this.setPieceJeuPlateau(pieceJeu, coordonnesInt[0], coordonnesInt[1]);
		}
	}*/
	
	private void setPieceJeuPlateau(PieceJeu pieceJeu, int x, int y){
		if ( x >= 0 && x <= 7 && y >= 0 && y <= 7 ){
			plateau[x][y] = pieceJeu;
		}
	}
	
	/*private PieceJeu getPieceJeuPlateau(String coordonnes){
		PieceJeu pieceJeu = null;
		int[] coordonnesInt = this.getCoordonnesIntPlateau(coordonnes);
		if ( coordonnesInt != null ){
			pieceJeu = this.getPieceJeuPlateau(coordonnesInt[0], coordonnesInt[1]);
		}
		return pieceJeu;
	}*/
	
	private PieceJeu getPieceJeuPlateau(int x, int y){
		PieceJeu pieceJeu = null;
		if ( x >= 0 && x <= 7 && y >= 0 && y <= 7 ){
			pieceJeu = plateau[x][y];
		}
		return pieceJeu;
	}
	
	private int[] getCoordonnesIntPlateau(String coordonnes){
		int[] coordonnesInt = null;
		if ( coordonnes != null ){
			if ( coordonnes.length() == 2 ){
				String xPlateau = coordonnes.toLowerCase().substring(0, 1);
				int yPlateau = -1;
				try{
					yPlateau = Integer.parseInt(coordonnes.substring(1, 2));
				}
				catch(Exception e){
					
				}
				int x = -1;
				switch(xPlateau){
					case "a":
						x = 0;
						break;
					case "b":
						x = 1;
						break;
					case "c":
						x = 2;
						break;
					case "d":
						x = 3;
						break;
					case "e":
						x = 4;
						break;
					case "f":
						x = 5;
						break;
					case "g":
						x = 6;
						break;
					case "h":
						x = 7;
						break;
				}
				int y = yPlateau - 1;
				if ( x >= 0 && x <= 7 && y >= 0 && y <= 7 ){
					coordonnesInt = new int[]{ x, y };
				}
			}
		}
		return coordonnesInt;
	}
	
	private void lignePlateau(int typeLigne, int idLigne) throws TransmissionException{
		this.lignePlateau(typeLigne, idLigne, false);
	}
	
	private void lignePlateau(int typeLigne, int idLigne, boolean afficherCoordonnesLettres) throws TransmissionException{
		if ( afficherCoordonnesLettres || idLigne >= 0 ){
			minitelTools.setColorText(CodeCouleurCaractere.caractereGris1);
		}
		for ( int i = 0 ; i < 8 ; i++ ){
			if ( i == 0 || i == 2 || i == 4 || i == 6 ){
				if ( typeLigne == 0 ){
					minitelTools.setColorBackground(CodeCouleurDeFond.fondGris4);
				}
				else{
					minitelTools.setColorBackground(CodeCouleurDeFond.fondGris2);
				}
			}
			else{
				if ( typeLigne == 0 ){
					minitelTools.setColorBackground(CodeCouleurDeFond.fondGris2);
				}
				else{
					minitelTools.setColorBackground(CodeCouleurDeFond.fondGris4);
				}
			}
			if ( afficherCoordonnesLettres ){
				minitelTools.write(" ");
				if ( i == 0 ){
					minitelTools.write("a");
				}
				if ( i == 1 ){
					minitelTools.write("b");
				}
				if ( i == 2 ){
					minitelTools.write("c");
				}
				if ( i == 3 ){
					minitelTools.write("d");
				}
				if ( i == 4 ){
					minitelTools.write("e");
				}
				if ( i == 5 ){
					minitelTools.write("f");
				}
				if ( i == 6 ){
					minitelTools.write("g");
				}
				if ( i == 7 ){
					minitelTools.write("h");
				}
				minitelTools.write(" ");
			}
			else{
				if ( idLigne >= 0 && i == 7 ){
					minitelTools.write("  " + idLigne);
				}
				else{
					minitelTools.write("   ");
				}
			}
		}
	}
}
